@@ -35,7 +35,6 @@
 #include "percussion_state.h"
 #include "ViewState.h"
 #include "UiSettings.h"
-#include "GeonkickConfig.h"
 
 #include <RkEvent.h>
 
@@ -51,15 +50,14 @@ MainWindow::MainWindow(RkMain *app, GeonkickApi *api, const std::string &preset)
         , limiterWidget{nullptr}
         , kitModel{new KitModel(this, geonkickApi)}
 {
-        GeonkickConfig config;
-        setScaleFactor(config.getScaleFactor());
+        setName("MainWindow");
+        setScaleFactor(geonkickApi->getScaleFactor());
         createViewState();
         setFixedSize(MAIN_WINDOW_WIDTH, MAIN_WINDOW_HEIGHT);
         setTitle(GEONKICK_NAME);
         geonkickApi->registerCallbacks(true);
-	RK_ACT_BIND(geonkickApi, stateChanged, RK_ACT_ARGS(), this, updateGui());
+        RK_ACT_BIND(geonkickApi, stateChanged, RK_ACT_ARGS(), this, updateGui());
         createShortcuts();
-        show();
 }
 
 MainWindow::MainWindow(RkMain *app, GeonkickApi *api, const RkNativeWindowInfo &info)
@@ -71,15 +69,13 @@ MainWindow::MainWindow(RkMain *app, GeonkickApi *api, const RkNativeWindowInfo &
         , limiterWidget{nullptr}
         , kitModel{new KitModel(this, geonkickApi)}
 {
-        GeonkickConfig config;
-        setScaleFactor(config.getScaleFactor());
+        setScaleFactor(geonkickApi->getScaleFactor());
         createViewState();
         setFixedSize(MAIN_WINDOW_WIDTH, MAIN_WINDOW_HEIGHT);
         setTitle(GEONKICK_NAME);
         geonkickApi->registerCallbacks(true);
         RK_ACT_BIND(geonkickApi, stateChanged, RK_ACT_ARGS(), this, updateGui());
         createShortcuts();
-        show();
 }
 
 MainWindow::~MainWindow()
@@ -174,8 +170,10 @@ bool MainWindow::init(void)
                     this, openFileDialog(FileDialog::Type::Save));
         RK_ACT_BIND(topBar, resetToDefault, RK_ACT_ARGS(),
                     this, resetToDefault());
+#ifndef GEONKICK_OS_WINDOWS
         RK_ACT_BIND(topBar, openExport, RK_ACT_ARGS(),
                     this, openExportDialog());
+#endif // GEONKICK_OS_WINDOWS
         RK_ACT_BIND(topBar, layerSelected,
                     RK_ACT_ARGS(GeonkickApi::Layer layer, bool b),
                     geonkickApi, enbaleLayer(layer, b));
@@ -186,6 +184,7 @@ bool MainWindow::init(void)
         envelopeWidget->setY(topBar->y() + topBar->height());
         envelopeWidget->setFixedSize(850, 305);
         envelopeWidget->show();
+        
         RK_ACT_BIND(this, updateGui, RK_ACT_ARGS(), envelopeWidget, updateGui());
         RK_ACT_BIND(envelopeWidget, requestUpdateGui, RK_ACT_ARGS(), this, updateGui());
         limiterWidget = new Limiter(geonkickApi, this);
@@ -212,14 +211,17 @@ bool MainWindow::init(void)
                 openPreset(presetName);
         topBar->setPresetName(geonkickApi->getPercussionName(geonkickApi->currentPercussion()));
         updateGui();
+        show();
         return true;
 }
 
+#ifndef GEONKICK_OS_WINDOWS
 void MainWindow::openExportDialog()
 {
         auto w = new ExportWidget(this, geonkickApi);
         w->setPosition(30, 40);
 }
+#endif // GEONKICK_OS_WINDOWS
 
 void MainWindow::savePreset(const std::string &fileName)
 {
@@ -267,7 +269,7 @@ void MainWindow::openPreset(const std::string &fileName)
         action geonkickApi->percussionUpdated(state->getId());
         file.close();
         geonkickApi->setCurrentWorkingPath("OpenPreset",
-                                           filePath.has_parent_path() ? filePath.parent_path() : filePath);
+                                           filePath.has_parent_path() ? filePath.parent_path().string() : filePath.string());
         updateGui();
 }
 
@@ -278,14 +280,14 @@ void MainWindow::openFileDialog(FileDialog::Type type)
         fileDialog->setFilters({".gkick", ".GKICK"});
         fileDialog->setHomeDirectory(geonkickApi->getSettings("GEONKICK_CONFIG/HOME_PATH"));
         if (type == FileDialog::Type::Open) {
-                fileDialog->setCurrentDirectoy(geonkickApi->currentWorkingPath("OpenPreset"));
+                fileDialog->setCurrentDirectoy(geonkickApi->currentWorkingPath("OpenPreset").string());
                 RK_ACT_BIND(fileDialog,
                             selectedFile,
                             RK_ACT_ARGS(const std::string &file),
                             this,
                             openPreset(file));
         } else {
-                fileDialog->setCurrentDirectoy(geonkickApi->currentWorkingPath("SavePreset"));
+                fileDialog->setCurrentDirectoy(geonkickApi->currentWorkingPath("SavePreset").string());
                 RK_ACT_BIND(fileDialog,
                             selectedFile,
                             RK_ACT_ARGS(const std::string &file),
@@ -311,7 +313,9 @@ void MainWindow::shortcutEvent(RkKeyEvent *event)
                         openFileDialog(FileDialog::Type::Save);
                 } else if (event->modifiers() & static_cast<int>(Rk::KeyModifiers::Control)
                            && (event->key() == Rk::Key::Key_e || event->key() == Rk::Key::Key_E)) {
+#ifndef GEONKICK_OS_WINDOWS
                         openExportDialog();
+#endif // GEONKICK_OS_WINDOWS
                 } else if ((event->modifiers() & static_cast<int>(Rk::KeyModifiers::Control))
                            && (event->key() == Rk::Key::Key_c || event->key() == Rk::Key::Key_C)) {
                         geonkickApi->copyToClipboard();
@@ -322,13 +326,11 @@ void MainWindow::shortcutEvent(RkKeyEvent *event)
                         updateGui();
                 } else if ((event->modifiers() & static_cast<int>(Rk::KeyModifiers::Control))
                            && (event->key() == Rk::Key::Key_F || event->key() == Rk::Key::Key_f)) {
-                        setScaleFactor((scaleFactor() + 0.5 > 2.1) ? 1 : scaleFactor() + 0.5);
+                        geonkickApi->setScaleFactor((geonkickApi->getScaleFactor() + 0.5 > 2.1) ? 1 : scaleFactor() + 0.5);
+                        setScaleFactor(geonkickApi->getScaleFactor());
                         setFixedSize(MAIN_WINDOW_WIDTH, MAIN_WINDOW_HEIGHT);
                         updateGui();
-                        GeonkickConfig config;
-                        config.setScaleFactor(scaleFactor());
-                        config.save();
-                        action onScaleFactor(scaleFactor());
+                        action onScaleFactor(geonkickApi->getScaleFactor());
                 }
 
                 if (event->modifiers() & static_cast<int>(Rk::KeyModifiers::Control_Left))
@@ -365,7 +367,7 @@ void MainWindow::dropEvent(RkDropEvent *event)
         std::string fileExtention;
         try {
                 std::filesystem::path path(event->getFilePath());
-                fileExtention = path.extension();
+                fileExtention = path.extension().string();
         } catch (const std::exception& e) {
                 GEONKICK_LOG_ERROR("can't create path " << e.what());
                 return;
@@ -402,3 +404,8 @@ void MainWindow::updateLimiter(KitModel::PercussionIndex index)
         if (kitModel->isPercussionSelected(index))
                 limiterWidget->onUpdateLimiter();
 }
+
+RkSize MainWindow::getWindowSize()
+{
+        return RkSize(MAIN_WINDOW_WIDTH, MAIN_WINDOW_HEIGHT);
+}
