@@ -28,6 +28,9 @@
 #include "envelope.h"
 #include "mixer.h"
 
+#include <unistd.h>
+#include <stdlib.h>
+
 enum geonkick_error
 geonkick_create(struct geonkick **kick, int sample_rate)
 {
@@ -56,7 +59,7 @@ geonkick_create(struct geonkick **kick, int sample_rate)
 
         (*kick)->sample_rate = (*kick)->audio->sample_rate;
 
-        for (size_t i = 0; i < GEONKICK_MAX_PERCUSSIONS; i++) {
+        for (size_t i = 0; i < GEONKICK_MAX_INSTRUMENTS; i++) {
                 if (gkick_synth_new(&(*kick)->synths[i], (*kick)->sample_rate) != GEONKICK_OK) {
                         gkick_log_error("can't create synthesizer %u", i);
                         geonkick_free(kick);
@@ -65,7 +68,7 @@ geonkick_create(struct geonkick **kick, int sample_rate)
                 (*kick)->synths[i]->id = i;
         }
 
-        for (size_t i = 0; i < GEONKICK_MAX_PERCUSSIONS; i++) {
+        for (size_t i = 0; i < GEONKICK_MAX_INSTRUMENTS; i++) {
                 gkick_synth_set_output((*kick)->synths[i], (*kick)->audio->audio_outputs[i]);
                 geonkick_set_percussion_channel(*kick, i, i % GEONKICK_MAX_CHANNELS);
         }
@@ -100,7 +103,7 @@ void geonkick_free(struct geonkick **kick)
                 if (geonkick_worker_reference_count() == 0)
                         geonkick_worker_destroy();
                 gkick_log_debug("ref count: %d", geonkick_worker_reference_count());
-                for (size_t i = 0; i < GEONKICK_MAX_PERCUSSIONS; i++)
+                for (size_t i = 0; i < GEONKICK_MAX_INSTRUMENTS; i++)
                         gkick_synth_free(&((*kick)->synths[i]));
                 gkick_audio_free(&((*kick)->audio));
 		pthread_mutex_destroy(&(*kick)->lock);
@@ -268,6 +271,41 @@ geonkick_osc_envelope_update_point(struct geonkick *kick,
         return res;
 }
 
+enum geonkick_error
+geonkick_osc_envelope_set_apply_type(struct geonkick *kick, 
+				     size_t osc_index,
+				     size_t env_index,
+				     enum gkick_envelope_apply_type apply_type)
+{
+	if (kick == NULL) {
+                gkick_log_error("wrong arguments");
+                return GEONKICK_ERROR;
+        }
+
+        enum geonkick_error res;
+        res = synth_osc_env_set_apply_type(kick->synths[kick->per_index],
+					   osc_index,
+					   env_index,
+					   apply_type);
+        if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
+                geonkick_wakeup(kick);
+        return res;
+}
+
+enum geonkick_error
+geonkick_osc_envelope_get_apply_type(struct geonkick *kick,
+				     size_t osc_index,
+				     size_t env_index,
+				     enum gkick_envelope_apply_type *apply_type)
+{
+        if (kick == NULL || apply_type == NULL)
+                return GEONKICK_ERROR;
+        return synth_osc_env_get_apply_type(kick->synths[kick->per_index],
+					    osc_index,
+					    env_index,
+					    apply_type);
+}
+
 enum geonkick_error
 geonkick_osc_set_fm(struct geonkick *kick, size_t index, bool is_fm)
 {
@@ -644,6 +682,39 @@ geonkick_kick_update_env_point(struct geonkick *kick,
         return res;
 }
 
+enum geonkick_error
+geonkick_kick_env_set_apply_type(struct geonkick *kick, 
+				 enum geonkick_envelope_type env_type,
+				 enum gkick_envelope_apply_type apply_type)
+{
+	if (kick == NULL) {
+                gkick_log_error("wrong arguments");
+                return GEONKICK_ERROR;
+        }
+        enum geonkick_error res;
+        res = synth_kick_env_set_apply_type(kick->synths[kick->per_index],
+					    env_type,
+                                            apply_type);
+        if (res == GEONKICK_OK && kick->synths[kick->per_index]->buffer_update)
+                geonkick_wakeup(kick);
+        return res;
+}
+
+enum geonkick_error
+geonkick_kick_env_get_apply_type(struct geonkick *kick,
+				 enum geonkick_envelope_type env_type,
+				 enum gkick_envelope_apply_type *apply_type)
+{
+	if (kick == NULL) {
+                gkick_log_error("wrong arguments");
+                return GEONKICK_ERROR;
+        }
+
+        return synth_kick_env_get_apply_type(kick->synths[kick->per_index],
+					     env_type,
+					     apply_type);
+}
+
 enum geonkick_error
 geonkick_set_osc_frequency(struct geonkick *kick,
                            size_t osc_index,
@@ -740,7 +811,7 @@ geonkick_get_osc_pitch_shift(struct geonkick *kick,
 enum geonkick_error
 geonkick_play(struct geonkick *kick, size_t id)
 {
-        if (kick == NULL || id >= GEONKICK_MAX_PERCUSSIONS) {
+        if (kick == NULL || id >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -806,7 +877,7 @@ geonkick_set_kick_buffer_callback(struct geonkick *kick,
         }
 
 	geonkick_lock(kick);
-        for (size_t i = 0; i < GEONKICK_MAX_PERCUSSIONS; i++) {
+        for (size_t i = 0; i < GEONKICK_MAX_INSTRUMENTS; i++) {
                 kick->synths[i]->buffer_callback = callback;
                 kick->synths[i]->callback_args = arg;
         }
@@ -1012,7 +1083,7 @@ geonkick_enable_synthesis(struct geonkick *kick,
 
 	kick->synthesis_on = enable;
         if (kick->synthesis_on) {
-                for (size_t i = 0; i < GEONKICK_MAX_PERCUSSIONS; i++) {
+                for (size_t i = 0; i < GEONKICK_MAX_INSTRUMENTS; i++) {
                         if (kick->synths[i]->is_active)
                                 kick->synths[i]->buffer_update = true;
                 }
@@ -1021,16 +1092,6 @@ geonkick_enable_synthesis(struct geonkick *kick,
         return GEONKICK_OK;
 }
 
-enum geonkick_error
-geonkick_get_audio_frame(struct geonkick *kick,
-                         int channel,
-                         gkick_real *val)
-{
-        return gkick_audio_get_frame(kick->audio,
-                                     channel,
-                                     val);
-}
-
 enum geonkick_error
 geonkick_audio_process(struct geonkick *kick,
                        float **out,
@@ -1463,7 +1524,7 @@ geonkick_is_audio_output_tuned(struct geonkick *kick,
                                bool *tune)
 {
         if (kick == NULL || tune == NULL
-            || index >= GEONKICK_MAX_PERCUSSIONS) {
+            || index >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1514,7 +1575,7 @@ enum geonkick_error
 geonkick_set_current_percussion(struct geonkick *kick,
                                 size_t index)
 {
-        if (kick == NULL || index >= GEONKICK_MAX_PERCUSSIONS) {
+        if (kick == NULL || index >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1539,7 +1600,7 @@ geonkick_get_current_percussion(struct geonkick *kick,
 
 void geonkick_process(struct geonkick *kick)
 {
-        for (size_t i = 0; i < GEONKICK_MAX_PERCUSSIONS; i++) {
+        for (size_t i = 0; i < GEONKICK_MAX_INSTRUMENTS; i++) {
                 struct gkick_synth *synth = kick->synths[i];
                 if (synth != NULL && synth->is_active && synth->buffer_update)
                         gkick_synth_process(synth);
@@ -1559,7 +1620,7 @@ geonkick_unused_percussion(struct geonkick *kick,
         }
 
 	*index = -1;
-        for (size_t i = 0; i < GEONKICK_MAX_PERCUSSIONS; i++) {
+        for (size_t i = 0; i < GEONKICK_MAX_INSTRUMENTS; i++) {
                 if (!kick->synths[i]->is_active) {
                         *index = i;
                         return GEONKICK_OK;
@@ -1573,7 +1634,7 @@ geonkick_enable_percussion(struct geonkick *kick,
                            size_t index,
                            bool enable)
 {
-        if (kick == NULL || index >= GEONKICK_MAX_PERCUSSIONS) {
+        if (kick == NULL || index >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1588,7 +1649,7 @@ geonkick_is_percussion_enabled(struct geonkick *kick,
                                bool *enable)
 {
         if (kick == NULL || enable == NULL
-            || index >= GEONKICK_MAX_PERCUSSIONS) {
+            || index >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1597,18 +1658,12 @@ geonkick_is_percussion_enabled(struct geonkick *kick,
         return GEONKICK_OK;
 }
 
-
-size_t geonkick_percussion_number()
-{
-	return GEONKICK_MAX_PERCUSSIONS;
-}
-
 enum geonkick_error
 geonkick_set_playing_key(struct geonkick *kick,
                          size_t id,
                          signed char key)
 {
-        if (kick == NULL || id >= GEONKICK_MAX_PERCUSSIONS) {
+        if (kick == NULL || id >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1619,7 +1674,7 @@ enum geonkick_error
 geonkick_get_playing_key(struct geonkick *kick, size_t id, signed char *key)
 {
         if (kick == NULL || key == NULL
-            || id >= GEONKICK_MAX_PERCUSSIONS) {
+            || id >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1627,13 +1682,72 @@ geonkick_get_playing_key(struct geonkick *kick, size_t id, signed char *key)
                                                   key);
 }
 
+enum geonkick_error
+geonkick_set_midi_channel(struct geonkick *kick,
+                          size_t id,
+                          signed char channel)
+{
+        if (kick == NULL || id >= GEONKICK_MAX_INSTRUMENTS) {
+                gkick_log_error("wrong arguments");
+                return GEONKICK_ERROR;
+        }
+        gkick_log_debug("id: %d, ch: %d", id, channel);
+        return gkick_audio_output_set_midi_channel(kick->synths[id]->output, channel);
+}
+
+enum geonkick_error
+geonkick_get_midi_channel(struct geonkick *kick,
+                          size_t id,
+                          signed char *channel)
+{
+        if (kick == NULL
+            || channel == NULL
+            || id >= GEONKICK_MAX_INSTRUMENTS) {
+                gkick_log_error("wrong arguments");
+                return GEONKICK_ERROR;
+        }
+        return gkick_audio_output_get_midi_channel(kick->synths[id]->output, channel);
+}
+
+enum geonkick_error
+geonkick_force_midi_channel(struct geonkick *kick,
+                            signed char channel,
+                            bool force)
+{
+        if (kick == NULL) {
+                gkick_log_error("wrong arguments");
+                return GEONKICK_ERROR;
+        }
+
+        return gkick_mixer_force_midi_channel(kick->audio->mixer, channel, force);
+}
+
+enum geonkick_error
+geonkick_ged_forced_midi_channel(struct geonkick *kick,
+                                 signed char *channel,
+                                 bool *force)
+{
+        if (kick == NULL) {
+                gkick_log_error("wrong arguments");
+                return GEONKICK_ERROR;
+        }
+
+        return gkick_mixer_get_forced_midi_channel(kick->audio->mixer, channel, force);
+}
+
+size_t
+geonkick_midi_channels_number(struct geonkick *kick)
+{
+        return GEONKICK_MAX_MIDI_CHANNELS;
+}
+
 enum geonkick_error
 geonkick_set_percussion_name(struct geonkick *kick,
                              size_t id,
                              const char *name,
                              size_t size)
 {
-        if (kick == NULL || id >= GEONKICK_MAX_PERCUSSIONS
+        if (kick == NULL || id >= GEONKICK_MAX_INSTRUMENTS
             || name == NULL || size < 1) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
@@ -1655,7 +1769,7 @@ geonkick_get_percussion_name(struct geonkick *kick,
                              char *name,
                              size_t size)
 {
-        if (kick == NULL || id >= GEONKICK_MAX_PERCUSSIONS
+        if (kick == NULL || id >= GEONKICK_MAX_INSTRUMENTS
             || name == NULL || size < 1) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
@@ -1677,12 +1791,18 @@ geonkick_channels_number()
         return GEONKICK_MAX_CHANNELS;
 }
 
+size_t
+geonkick_instruments_number()
+{
+        return GEONKICK_MAX_INSTRUMENTS;
+}
+
 enum geonkick_error
 geonkick_set_percussion_channel(struct geonkick *kick,
                                 size_t id,
                                 size_t channel)
 {
-        if (kick == NULL || id >= GEONKICK_MAX_PERCUSSIONS
+        if (kick == NULL || id >= GEONKICK_MAX_INSTRUMENTS
             || channel >= GEONKICK_MAX_CHANNELS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
@@ -1695,7 +1815,7 @@ geonkick_get_percussion_channel(struct geonkick *kick,
                                 size_t id,
                                 size_t *channel)
 {
-        if (kick == NULL || channel == NULL || id >= GEONKICK_MAX_PERCUSSIONS) {
+        if (kick == NULL || channel == NULL || id >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1708,7 +1828,7 @@ geonkick_percussion_set_limiter(struct geonkick *kick,
                                 size_t id,
                                 gkick_real val)
 {
-        if (kick == NULL || id >= GEONKICK_MAX_PERCUSSIONS) {
+        if (kick == NULL || id >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1720,7 +1840,7 @@ geonkick_percussion_get_limiter(struct geonkick *kick,
                                 size_t id,
                                 gkick_real *val)
 {
-        if (kick == NULL || val == NULL || id >= GEONKICK_MAX_PERCUSSIONS) {
+        if (kick == NULL || val == NULL || id >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1732,7 +1852,7 @@ geonkick_percussion_mute(struct geonkick *kick,
                          size_t id,
                          bool b)
 {
-        if (kick == NULL || id >= GEONKICK_MAX_PERCUSSIONS) {
+        if (kick == NULL || id >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1744,7 +1864,7 @@ geonkick_percussion_is_muted(struct geonkick *kick,
                              size_t id,
                              bool *b)
 {
-        if (kick == NULL || b == NULL || id >= GEONKICK_MAX_PERCUSSIONS) {
+        if (kick == NULL || b == NULL || id >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1756,7 +1876,7 @@ geonkick_percussion_solo(struct geonkick *kick,
                          size_t id,
                          bool b)
 {
-        if (kick == NULL || id >= GEONKICK_MAX_PERCUSSIONS) {
+        if (kick == NULL || id >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1768,7 +1888,7 @@ geonkick_percussion_is_solo(struct geonkick *kick,
                             size_t id,
                             bool *b)
 {
-        if (kick == NULL || b == NULL || id >= GEONKICK_MAX_PERCUSSIONS) {
+        if (kick == NULL || b == NULL || id >= GEONKICK_MAX_INSTRUMENTS) {
                 gkick_log_error("wrong arguments");
                 return GEONKICK_ERROR;
         }
@@ -1785,7 +1905,8 @@ geonkick_set_preview_sample(struct geonkick *kick,
                 return GEONKICK_ERROR;
         }
 
-        struct gkick_audio_output *output = kick->audio->mixer->audio_outputs[GEONKICK_MAX_PERCUSSIONS];
+        struct gkick_audio_output *output;
+        output = kick->audio->mixer->audio_outputs[GEONKICK_AUDITION_CHANNEL_INDEX];
         gkick_audio_output_lock(output);
         gkick_buffer_set_data((struct gkick_buffer*)output->updated_buffer, data, size);
         gkick_audio_output_unlock(output);
@@ -1795,19 +1916,24 @@ geonkick_set_preview_sample(struct geonkick *kick,
 void
 geonkick_play_sample_preview(struct geonkick *kick)
 {
-        gkick_audio_play(kick->audio, GEONKICK_MAX_PERCUSSIONS);
+        gkick_audio_play(kick->audio,
+                         GEONKICK_AUDITION_CHANNEL_INDEX);
 }
 
 enum geonkick_error
 geonkick_set_sample_preview_limiter(struct geonkick *kick, gkick_real val)
 {
-        return gkick_audio_set_limiter_val(kick->audio, GEONKICK_MAX_PERCUSSIONS, val);
+        return gkick_audio_set_limiter_val(kick->audio,
+                                           GEONKICK_AUDITION_CHANNEL_INDEX,
+                                           val);
 }
 
 enum geonkick_error
 geonkick_get_sample_preview_limiter(struct geonkick *kick, gkick_real *val)
 {
-        return gkick_audio_get_limiter_val(kick->audio, GEONKICK_MAX_PERCUSSIONS, val);
+        return gkick_audio_get_limiter_val(kick->audio,
+                                           GEONKICK_AUDITION_CHANNEL_INDEX,
+                                           val);
 }
 
 void
@@ -1822,3 +1948,22 @@ geonkick_is_plugin()
 {
         return !GKICK_IS_STANDALONE;
 }
+
+void
+geonkick_usleep(unsigned long int usecods)
+{
+        usleep(usecods);
+}
+
+
+int geonkick_rand(unsigned int *seed)
+{
+#ifdef GEONKICK_OS_WINDOWS
+        GEONKICK_UNUSED(seed);
+        // Use rand & srand since the calls are only from a single thread.
+        return rand();
+#else
+        return rand_r(seed);
+#endif // GEONKICK_OS_GNU
+}
+
