@@ -30,7 +30,7 @@ extern "C" {
 
 #include <stdio.h>
 #include <errno.h>
-#include <unistd.h>
+//#include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
@@ -39,6 +39,10 @@ extern "C" {
 #include <float.h>
 #include <stdbool.h>
 
+#ifdef __FAST_MATH__
+#error -ffast-math disables nan detection needed by geonkick
+#endif
+
 #ifdef __STDC_NO_ATOMICS__
 #error atomic operations are not supported
 #endif
@@ -53,10 +57,10 @@ typedef float gkick_real;
 
 #define GEONKICK_UNUSED(expr) (void)expr
 
-#define GEONKICK_VERSION 0x021002
+#define GEONKICK_VERSION GEONKICK_VERSION_HEX
 #define GEONKICK_NAME "Geonkick"
 #define GEONKICK_APP_NAME "geonkick"
-#define GEOKICK_VERSION_STRING "2.10.2"
+#define GEOKICK_VERSION_STRING GEONKICK_VERSION_STR
 
 #ifdef GEONKICK_AUDIO_JACK
 #define GKICK_IS_STANDALONE (1)
@@ -80,12 +84,13 @@ enum gkick_key_state {
 };
 
 enum geonkick_envelope_type {
-        GEONKICK_AMPLITUDE_ENVELOPE = 0,
-        GEONKICK_FREQUENCY_ENVELOPE = 1,
-        GEONKICK_FILTER_CUTOFF_ENVELOPE = 2,
-	GEONKICK_DISTORTION_DRIVE_ENVELOPE = 3,
+        GEONKICK_AMPLITUDE_ENVELOPE         = 0,
+        GEONKICK_FREQUENCY_ENVELOPE         = 1,
+        GEONKICK_FILTER_CUTOFF_ENVELOPE     = 2,
+	GEONKICK_DISTORTION_DRIVE_ENVELOPE  = 3,
         GEONKICK_DISTORTION_VOLUME_ENVELOPE = 4,
-        GEONKICK_PITCH_SHIFT_ENVELOPE = 5
+        GEONKICK_PITCH_SHIFT_ENVELOPE       = 5,
+	GEONKICK_FILTER_Q_ENVELOPE          = 6,
 };
 
 enum gkick_filter_type {
@@ -120,22 +125,23 @@ enum geonkick_channel_type {
         GEONKICK_CHANNEL_MIDI_OUTPUT  = 3
 };
 
+enum gkick_envelope_apply_type {
+	GEONKICK_ENVELOPE_APPLY_LINEAR       = 0,
+	GEONKICK_ENVELOPE_APPLY_LOGARITHMIC  = 1
+};
+
 #ifdef GEONKICK_SINGLE
-#define GEONKICK_MAX_PERCUSSIONS 1
+#define GEONKICK_MAX_INSTRUMENTS 1
 #else
-#define GEONKICK_MAX_PERCUSSIONS 16
+#define GEONKICK_MAX_INSTRUMENTS 16
 #endif
 
-/**
-* Maximum audio number of output stereo channels.
-*/
-#if defined(GEONKICK_SINGLE)
-#define GEONKICK_MAX_CHANNELS 1
-#else
-#define GEONKICK_MAX_CHANNELS 16
-#endif // if defined(GEONKICK_SINGLE)
+#define GEONKICK_MAX_CHANNELS (GEONKICK_MAX_INSTRUMENTS + 1)
+#define GEONKICK_AUDITION_CHANNEL_INDEX (GEONKICK_MAX_CHANNELS - 1)
 
 #define GEONKICK_ANY_KEY -1
+#define GEONKICK_ANY_MIDI_CHANNEL -1
+#define GEONKICK_MAX_MIDI_CHANNELS 16
 
 struct geonkick;
 
@@ -204,6 +210,18 @@ geonkick_osc_envelope_update_point(struct geonkick *kick,
 				   gkick_real x,
 				   gkick_real y);
 
+enum geonkick_error
+geonkick_osc_envelope_set_apply_type(struct geonkick *kick, 
+				     size_t osc_index,
+				     size_t env_index,
+				     enum gkick_envelope_apply_type apply_type);
+
+enum geonkick_error
+geonkick_osc_envelope_get_apply_type(struct geonkick *kick,
+				     size_t osc_index,
+				     size_t env_index,
+				     enum gkick_envelope_apply_type *apply_type);
+
 enum geonkick_error
 geonkick_osc_set_fm(struct geonkick *kick,
                     size_t index,
@@ -325,6 +343,16 @@ geonkick_kick_update_env_point(struct geonkick *kick,
                                gkick_real x,
                                gkick_real y);
 
+enum geonkick_error
+geonkick_kick_env_set_apply_type(struct geonkick *kick, 
+				 enum geonkick_envelope_type env_type,
+				 enum gkick_envelope_apply_type apply_type);
+
+enum geonkick_error
+geonkick_kick_env_get_apply_type(struct geonkick *kick,
+				 enum geonkick_envelope_type env_type,
+				 enum gkick_envelope_apply_type *apply_type);
+
 enum geonkick_error
 geonkick_set_osc_amplitude(struct geonkick *kick,
                            size_t osc_index,
@@ -444,11 +472,6 @@ enum geonkick_error
 geonkick_enable_synthesis(struct geonkick *kick,
                           bool enable);
 
-enum geonkick_error
-geonkick_get_audio_frame(struct geonkick *kick,
-                         int channel,
-                         gkick_real *val);
-
 enum geonkick_error
 geonkick_audio_process(struct geonkick *kick,
                        float **out,
@@ -610,9 +633,6 @@ geonkick_is_percussion_enabled(struct geonkick *kick,
                                size_t index,
                                bool *enable);
 
-size_t
-geonkick_percussion_number();
-
 enum geonkick_error
 geonkick_set_playing_key(struct geonkick *kick,
                          size_t id,
@@ -623,6 +643,29 @@ geonkick_get_playing_key(struct geonkick *kick,
                          size_t id,
                          signed char *key);
 
+enum geonkick_error
+geonkick_set_midi_channel(struct geonkick *kick,
+                          size_t id,
+                          signed char channel);
+
+enum geonkick_error
+geonkick_get_midi_channel(struct geonkick *kick,
+                          size_t id,
+                          signed char *channel);
+
+enum geonkick_error
+geonkick_force_midi_channel(struct geonkick *kick,
+                            signed char channel,
+                            bool force);
+
+enum geonkick_error
+geonkick_ged_forced_midi_channel(struct geonkick *kick,
+                                 signed char *channel,
+                                 bool *force);
+        
+size_t
+geonkick_midi_channels_number();
+
 enum geonkick_error
 geonkick_set_percussion_name(struct geonkick *kick,
                              size_t id,
@@ -635,6 +678,10 @@ geonkick_get_percussion_name(struct geonkick *kick,
                              char *name,
                              size_t size);
 
+
+size_t
+geonkick_instruments_number();
+
 size_t
 geonkick_channels_number();
 
