@@ -43,7 +43,6 @@ gkick_synth_new(struct gkick_synth **synth, int sample_rate)
 	(*synth)->oscillators_number = GKICK_OSC_GROUPS_NUMBER * GKICK_OSC_GROUP_SIZE;
         (*synth)->buffer_update = 0;
         (*synth)->amplitude = 1.0f;
-        (*synth)->buffer_size = (size_t)((*synth)->length * (*synth)->sample_rate);
         (*synth)->buffer_update = false;
         (*synth)->is_active = false;
         memset((*synth)->name, '\0', sizeof((*synth)->name));
@@ -93,8 +92,8 @@ gkick_synth_new(struct gkick_synth **synth, int sample_rate)
                 gkick_log_error("can't create synthesizer kick buffer");
                 gkick_synth_free(synth);
         }
-        gkick_buffer_set_size(buff, (*synth)->buffer_size);
-        (*synth)->buffer = (char*)buff;
+        gkick_buffer_set_size(buff, (size_t)((*synth)->length * (*synth)->sample_rate));
+        (*synth)->buffer = buff;
 
         if (gkick_synth_create_oscillators(*synth) != GEONKICK_OK) {
                 gkick_log_error("can't create oscillators");
@@ -114,10 +113,8 @@ void gkick_synth_free(struct gkick_synth **synth)
                         free((*synth)->oscillators);
                         (*synth)->oscillators = NULL;
 
-                        if ((*synth)->buffer != NULL) {
-                                free((*synth)->buffer);
-                                (*synth)->buffer = NULL;
-                        }
+                        if ((*synth)->buffer != NULL)
+                                gkick_buffer_free(&(*synth)->buffer);
 
                         if ((*synth)->filter)
                                 gkick_filter_free(&(*synth)->filter);
@@ -496,6 +493,66 @@ gkick_synth_osc_env_update_point(struct gkick_synth *synth,
         return GEONKICK_OK;
 }
 
+enum geonkick_error
+synth_osc_env_set_apply_type(struct gkick_synth *synth, 
+			     size_t osc_index,
+			     size_t env_index,
+			     enum gkick_envelope_apply_type apply_type)
+{
+	if (synth == NULL) {
+                gkick_log_error("wrong arguments");
+                return GEONKICK_ERROR;
+        }
+
+        gkick_synth_lock(synth);
+	        struct gkick_oscillator *osc = gkick_synth_get_oscillator(synth, osc_index);
+        if (osc == NULL) {
+                 gkick_log_error("can't get oscillator %d", osc_index);
+                 gkick_synth_unlock(synth);
+                 return GEONKICK_ERROR;
+        }
+
+        struct gkick_envelope *env = gkick_osc_get_envelope(osc, env_index);
+        if (env == NULL) {
+                gkick_synth_unlock(synth);
+                gkick_log_error("can't get envelope");
+                return GEONKICK_ERROR;
+        }
+        gkick_envelope_set_apply_type(env, apply_type);
+	osc = gkick_synth_get_oscillator(synth, osc_index);
+        if (synth->osc_groups[osc_index / GKICK_OSC_GROUP_SIZE]
+            && osc->state == GEONKICK_OSC_STATE_ENABLED) {
+                synth->buffer_update = true;
+        }
+        gkick_synth_unlock(synth);
+
+        return GEONKICK_OK;
+}
+
+enum geonkick_error
+synth_osc_env_get_apply_type(struct gkick_synth *synth,
+			     size_t osc_index,
+			     size_t env_index,
+			     enum gkick_envelope_apply_type *apply_type)
+{
+	if (synth == NULL) {
+                gkick_log_error("wrong arguments");
+                return GEONKICK_ERROR;
+        }
+
+        gkick_synth_lock(synth);
+	struct gkick_envelope *env = gkick_synth_osc_get_env(synth, osc_index, env_index);
+        if (env == NULL) {
+                gkick_synth_unlock(synth);
+                gkick_log_error("can't get envelope %d", env_index);
+                return GEONKICK_ERROR;
+        }
+	*apply_type = gkick_envelope_get_apply_type(env);
+        gkick_synth_unlock(synth);
+
+        return GEONKICK_OK;
+}
+
 enum geonkick_error
 gkick_synth_set_osc_function(struct gkick_synth *synth,
                              size_t osc_index,
@@ -687,7 +744,7 @@ gkick_synth_set_length(struct gkick_synth *synth,
 
         gkick_synth_lock(synth);
         synth->length = len;
-        synth->buffer_size = synth->length * synth->sample_rate;
+        gkick_buffer_set_size(synth->buffer, synth->length * synth->sample_rate);
         synth->buffer_update = true;
         gkick_synth_unlock(synth);
 
@@ -847,6 +904,26 @@ gkick_synth_get_kick_filter_type(struct gkick_synth *synth,
         return gkick_filter_get_type(synth->filter, type);
 }
 
+struct gkick_envelope*
+synth_get_kick_envelope(struct gkick_synth *synth,
+			enum geonkick_envelope_type env_type)
+{
+	switch(env_type) {
+	case GEONKICK_AMPLITUDE_ENVELOPE:
+                return synth->envelope;
+	case GEONKICK_FILTER_CUTOFF_ENVELOPE:
+                return synth->filter->cutoff_env;
+	case GEONKICK_FILTER_Q_ENVELOPE:
+                return synth->filter->q_env;
+	case GEONKICK_DISTORTION_DRIVE_ENVELOPE:
+		return synth->distortion->drive_env;
+	case GEONKICK_DISTORTION_VOLUME_ENVELOPE:
+                return synth->distortion->volume_env;
+	default:
+		return NULL;
+	}
+}
+
 enum geonkick_error
 gkick_synth_kick_envelope_get_points(struct gkick_synth *synth,
                                      enum geonkick_envelope_type env_type,
@@ -860,27 +937,11 @@ gkick_synth_kick_envelope_get_points(struct gkick_synth *synth,
 
         *npoints = 0;
         *buf = NULL;
-
         gkick_synth_lock(synth);
-        if (env_type == GEONKICK_AMPLITUDE_ENVELOPE)
-                gkick_envelope_get_points(synth->envelope,
-                                          buf,
-                                          npoints);
-        else if (env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE)
-                gkick_envelope_get_points(synth->filter->cutoff_env,
-                                          buf,
-                                          npoints);
-	else if (env_type == GEONKICK_DISTORTION_DRIVE_ENVELOPE)
-		gkick_envelope_get_points(synth->distortion->drive_env,
-                                          buf,
-                                          npoints);
-        else if (env_type == GEONKICK_DISTORTION_VOLUME_ENVELOPE)
-                gkick_envelope_get_points(synth->distortion->volume_env,
-                                          buf,
-                                          npoints);
-
+	struct gkick_envelope *env = synth_get_kick_envelope(synth, env_type);
+	if (env != NULL)
+		gkick_envelope_get_points(env, buf, npoints);
         gkick_synth_unlock(synth);
-
         return GEONKICK_OK;
 }
 
@@ -896,26 +957,13 @@ gkick_synth_kick_envelope_set_points(struct gkick_synth *synth,
         }
 
         gkick_synth_lock(synth);
-        if (env_type == GEONKICK_AMPLITUDE_ENVELOPE)
-                gkick_envelope_set_points(synth->envelope,
-                                          buf,
-                                          npoints);
-        else if (env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE)
-                gkick_envelope_set_points(synth->filter->cutoff_env,
-                                          buf,
-                                          npoints);
-	else if (env_type == GEONKICK_DISTORTION_DRIVE_ENVELOPE)
-		gkick_envelope_set_points(synth->distortion->drive_env,
-                                          buf,
-                                          npoints);
-        else if (env_type == GEONKICK_DISTORTION_VOLUME_ENVELOPE)
-		gkick_envelope_set_points(synth->distortion->volume_env,
-                                          buf,
-                                          npoints);
-
+	struct gkick_envelope *env = synth_get_kick_envelope(synth, env_type);
+	if (env != NULL)
+		gkick_envelope_set_points(env, buf, npoints);
 
 	if (env_type == GEONKICK_AMPLITUDE_ENVELOPE
-            || (env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE
+            || ((env_type == GEONKICK_FILTER_Q_ENVELOPE
+		 || env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE)
                 && synth->filter_enabled)
 	    || ((env_type == GEONKICK_DISTORTION_DRIVE_ENVELOPE
                  || env_type == GEONKICK_DISTORTION_VOLUME_ENVELOPE)
@@ -938,18 +986,13 @@ gkick_synth_kick_add_env_point(struct gkick_synth *synth,
         }
 
         gkick_synth_lock(synth);
-        if (env_type == GEONKICK_AMPLITUDE_ENVELOPE)
-                gkick_envelope_add_point(synth->envelope, x, y);
-        else if (env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE)
-                gkick_envelope_add_point(synth->filter->cutoff_env, x, y);
-	else if (env_type == GEONKICK_DISTORTION_DRIVE_ENVELOPE)
-		gkick_envelope_add_point(synth->distortion->drive_env, x, y);
-        else if (env_type == GEONKICK_DISTORTION_VOLUME_ENVELOPE)
-		gkick_envelope_add_point(synth->distortion->volume_env, x, y);
-
+	struct gkick_envelope *env = synth_get_kick_envelope(synth, env_type);
+	if (env != NULL)
+		gkick_envelope_add_point(env, x, y);
 
         if (env_type == GEONKICK_AMPLITUDE_ENVELOPE
-            || (env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE
+            || ((env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE
+		 || env_type == GEONKICK_FILTER_Q_ENVELOPE)
                 && synth->filter_enabled)
 	    || ((env_type == GEONKICK_DISTORTION_DRIVE_ENVELOPE
                  || env_type == GEONKICK_DISTORTION_VOLUME_ENVELOPE)
@@ -970,18 +1013,13 @@ gkick_synth_kick_remove_env_point(struct gkick_synth *synth,
         }
 
         gkick_synth_lock(synth);
-        if (env_type == GEONKICK_AMPLITUDE_ENVELOPE)
-                gkick_envelope_remove_point(synth->envelope, index);
-        else if (env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE)
-                gkick_envelope_remove_point(synth->filter->cutoff_env, index);
-	else if (env_type == GEONKICK_DISTORTION_DRIVE_ENVELOPE)
-		gkick_envelope_remove_point(synth->distortion->drive_env, index);
-        else if (env_type == GEONKICK_DISTORTION_VOLUME_ENVELOPE)
-		gkick_envelope_remove_point(synth->distortion->volume_env, index);
-
+	struct gkick_envelope *env = synth_get_kick_envelope(synth, env_type);
+	if (env != NULL)
+		gkick_envelope_remove_point(env, index);
 
         if (env_type == GEONKICK_AMPLITUDE_ENVELOPE
-            || (env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE
+            || ((env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE
+		|| env_type == GEONKICK_FILTER_Q_ENVELOPE)
                 && synth->filter_enabled)
 	    || ((env_type == GEONKICK_DISTORTION_DRIVE_ENVELOPE
                  || env_type == GEONKICK_DISTORTION_VOLUME_ENVELOPE)
@@ -1005,25 +1043,13 @@ gkick_synth_kick_update_env_point(struct gkick_synth *synth,
         }
 
         gkick_synth_lock(synth);
-        if (env_type == GEONKICK_AMPLITUDE_ENVELOPE)
-                gkick_envelope_update_point(synth->envelope,
-                                            index,
-                                            x, y);
-        else if (env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE)
-                gkick_envelope_update_point(synth->filter->cutoff_env,
-                                            index,
-                                            x, y);
-	else if (env_type == GEONKICK_DISTORTION_DRIVE_ENVELOPE)
-		gkick_envelope_update_point(synth->distortion->drive_env,
-                                            index,
-                                            x, y);
-        else if (env_type == GEONKICK_DISTORTION_VOLUME_ENVELOPE)
-		gkick_envelope_update_point(synth->distortion->volume_env,
-                                            index,
-                                            x, y);
+	struct gkick_envelope *env = synth_get_kick_envelope(synth, env_type);
+	if (env != NULL)
+		gkick_envelope_update_point(env, index, x, y);
 
         if (env_type == GEONKICK_AMPLITUDE_ENVELOPE
-            || (env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE
+            || ((env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE
+		 || env_type == GEONKICK_FILTER_Q_ENVELOPE)
                 && synth->filter_enabled)
 	    || ((env_type == GEONKICK_DISTORTION_DRIVE_ENVELOPE
                  || env_type == GEONKICK_DISTORTION_VOLUME_ENVELOPE)
@@ -1034,6 +1060,46 @@ gkick_synth_kick_update_env_point(struct gkick_synth *synth,
         return GEONKICK_OK;
 }
 
+enum geonkick_error
+synth_kick_env_set_apply_type(struct gkick_synth *synth, 
+			      enum geonkick_envelope_type env_type,
+			      enum gkick_envelope_apply_type apply_type)
+{
+        gkick_synth_lock(synth);
+        if (env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE) {
+		gkick_envelope_set_apply_type(synth->filter->cutoff_env, apply_type);
+	}
+
+        if (env_type == GEONKICK_AMPLITUDE_ENVELOPE
+            || (env_type == GEONKICK_FILTER_CUTOFF_ENVELOPE
+                && synth->filter_enabled)
+	    || ((env_type == GEONKICK_DISTORTION_DRIVE_ENVELOPE
+                 || env_type == GEONKICK_DISTORTION_VOLUME_ENVELOPE)
+                && synth->distortion->enabled)) {
+                synth->buffer_update = true;
+        }
+        gkick_synth_unlock(synth);
+        return GEONKICK_OK;	
+}
+
+enum geonkick_error
+synth_kick_env_get_apply_type(struct gkick_synth *synth,
+			      enum geonkick_envelope_type env_type,
+			      enum gkick_envelope_apply_type *apply_type)
+{
+	gkick_synth_lock(synth);
+        switch (env_type) {
+        case GEONKICK_FILTER_CUTOFF_ENVELOPE:
+		*apply_type = gkick_envelope_get_apply_type(synth->filter->cutoff_env);
+                break;
+        default:
+                *apply_type = GEONKICK_ENVELOPE_APPLY_LINEAR;
+                break;
+        }
+        gkick_synth_unlock(synth);
+        return GEONKICK_OK;	
+}
+
 enum geonkick_error
 gkick_synth_set_osc_frequency(struct gkick_synth *synth,
                              size_t osc_index,
@@ -1199,7 +1265,7 @@ gkick_synth_get_buffer_size(struct gkick_synth *synth,
         }
 
         gkick_synth_lock(synth);
-        *size = synth->buffer_size;
+        *size = gkick_buffer_size(synth->buffer);
         gkick_synth_unlock(synth);
         return GEONKICK_OK;
 }
@@ -1215,14 +1281,8 @@ gkick_synth_get_buffer(struct gkick_synth *synth,
         }
 
         gkick_synth_lock(synth);
-
-        if (size >= synth->buffer_size)
-                memcpy(buffer, synth->buffer,
-                       synth->buffer_size * sizeof(gkick_real));
-        else
-                memcpy(buffer, synth->buffer,
-                       size * sizeof(gkick_real));
-
+        size = min(gkick_buffer_size(synth->buffer), size);
+        memcpy(buffer, synth->buffer->buff, size * sizeof(gkick_real));
         gkick_synth_unlock(synth);
 
         return GEONKICK_ERROR;
@@ -1247,9 +1307,8 @@ gkick_synth_process(struct gkick_synth *synth)
 
 	gkick_synth_lock(synth);
 	synth->buffer_update = false;
-	gkick_buffer_set_size((struct gkick_buffer*)synth->buffer,
-                              synth->buffer_size);
-	gkick_real dt = synth->length / synth->buffer_size;
+        gkick_buffer_set_size(synth->buffer, synth->length * synth->sample_rate);
+	gkick_real dt = synth->length / gkick_buffer_size(synth->buffer);
 	gkick_synth_reset_oscillators(synth);
 	gkick_filter_init(synth->filter);
 	gkick_synth_unlock(synth);
@@ -1263,7 +1322,7 @@ gkick_synth_process(struct gkick_synth *synth)
                  * of the synthesizer parameters for too long time.
                  */
                 if (pthread_mutex_trylock(&synth->lock) != 0) {
-                        usleep(50);
+                        geonkick_usleep(50);
                         /**
                          * Check how many tries for locking to avoind infinite loop.
                          * It should be maximum around 30ms.
@@ -1285,7 +1344,7 @@ gkick_synth_process(struct gkick_synth *synth)
 				val = 1.0f;
 			else if (val < -1.0f)
 				val = -1.0f;
-			gkick_buffer_push_back((struct gkick_buffer*)synth->buffer, val);
+			gkick_buffer_push_back(synth->buffer, val);
 			i++;
 		}
                 gkick_synth_unlock(synth);
@@ -1293,9 +1352,10 @@ gkick_synth_process(struct gkick_synth *synth)
 
 	gkick_synth_lock(synth);
         if (synth->buffer_callback != NULL && synth->callback_args != NULL) {
+                gkick_log_debug("synth->buffer_callback");
                 synth->buffer_callback(synth->callback_args,
-                                       ((struct gkick_buffer*)synth->buffer)->buff,
-                                       synth->buffer_size,
+                                       synth->buffer->buff,
+                                       gkick_buffer_size(synth->buffer),
                                        synth->id);
         }
 
@@ -1306,7 +1366,7 @@ gkick_synth_process(struct gkick_synth *synth)
          */
         if (!synth->buffer_update) {
                 gkick_audio_output_lock(synth->output);
-                char* buff = synth->output->updated_buffer;
+                struct gkick_buffer *buff = synth->output->updated_buffer;
                 synth->output->updated_buffer = synth->buffer;
                 synth->buffer = buff;
                 gkick_audio_output_unlock(synth->output);
