@@ -25,7 +25,6 @@
 #include "globals.h"
 
 #include <iomanip>
-#include <math.h>
 
 Envelope::Envelope(const RkRect &area)
         : drawingArea{area}
@@ -33,11 +32,12 @@ Envelope::Envelope(const RkRect &area)
         , dotRadius{3}
         , selectedPointIndex{0}
         , supportedTypes({Type::Amplitude,
-			 Type::Frequency,
-			 Type::FilterCutOff,
-                         Type::DistortionDrive,
-                         Type::DistortionVolume,
-                         Type::PitchShift})
+			Type::Frequency,
+			Type::FilterCutOff,
+			Type::DistortionDrive,
+			Type::DistortionVolume,
+			Type::PitchShift,
+			Type::FilterQFactor})
         , overPointIndex{0}
         , isOverPoint{false}
         , pointSelected{false}
@@ -45,6 +45,7 @@ Envelope::Envelope(const RkRect &area)
         , envelopeType{Type::Amplitude}
         , editedPointIndex{0}
         , isEditingPoint{false}
+	, applyType{ApplyType::Linear}
 {
 }
 
@@ -137,6 +138,9 @@ void Envelope::drawValueScale(RkPainter &painter)
         case Type::FilterCutOff:
                 text = "Frequency, Hz";
                 break;
+	case Type::FilterQFactor:
+                text = "Resonance";
+                break;	
         case Type::PitchShift:
                 text = "Semitones";
                 break;
@@ -158,6 +162,7 @@ void Envelope::drawValueScale(RkPainter &painter)
         painter.setPen(RkPen(RkColor(110, 110, 110)));
 
         if (type() == Type::Amplitude
+	    || type() == Type::FilterQFactor
             || type() == Type::DistortionDrive
             || type() == Type::DistortionVolume) {
                 double step = envelopeAmplitude() / 10;
@@ -265,6 +270,7 @@ void Envelope::drawPoint(RkPainter &painter, const RkPoint &point)
 void Envelope::drawPointValue(RkPainter &painter, const RkPoint &point, double value)
 {
         if (type() == Envelope::Type::Amplitude
+	    || type() == Type::FilterQFactor
             || type() == Type::DistortionDrive
             || type() == Type::DistortionVolume) {
                 value *= envelopeAmplitude();
@@ -276,19 +282,36 @@ void Envelope::drawPointValue(RkPainter &painter, const RkPoint &point, double v
                 value = 2 * value - envelopeAmplitude();
                 painter.drawText(point.x(), point.y(), Geonkick::doubleToStr(value, 1));
         } else if (type() == Envelope::Type::Frequency || type() == Type::FilterCutOff) {
-                value *= envelopeAmplitude();
-                if (value < 20)
-                        painter.drawText(point.x(), point.y(), "20Hz " + frequencyToNote(20));
-                if (value >= 20 && value < 1000) {
-                        painter.drawText(point.x(), point.y(),
-                                         Geonkick::doubleToStr(value, 0)
-                                         + "Hz " + frequencyToNote(value));
-                } else if (value >= 1000 && value <= 20000) {
-                        painter.drawText(point.x(), point.y(),
-                                         Geonkick::doubleToStr(value / 1000, 0)
-                                         + std::string("kHz ")
-                                         + frequencyToNote(value));
-                }
+                if (getApplyType() == ApplyType::Linear) {
+			value *= envelopeAmplitude();
+			if (value < 20)
+				painter.drawText(point.x(), point.y(), "20Hz " + frequencyToNote(20));
+			if (value >= 20 && value < 1000) {
+				painter.drawText(point.x(), point.y(),
+						 Geonkick::doubleToStr(value, 0)
+						 + "Hz " + frequencyToNote(value));
+			} else if (value >= 1000 && value <= 20000) {
+				painter.drawText(point.x(), point.y(),
+						 Geonkick::doubleToStr(value / 1000, 0)
+						 + std::string("kHz ")
+						 + frequencyToNote(value));
+			}
+                } else {    
+			auto frequency =  pow(10, (log10(envelopeAmplitude()) - log10(20)) * value + log10(20));
+			if (frequency >= 20 && frequency < 1000) {
+				double roundedValue = std::round(frequency * 10.0) / 10.0;
+				painter.drawText(point.x(), point.y(),
+						 Geonkick::doubleToStr(roundedValue, 1)
+						 + "Hz " + frequencyToNote(frequency));
+			} else if (frequency >= 1000 && frequency <= 20000) {
+				frequency /= 1000;
+				double roundedValue = std::round(frequency * 10.0) / 10.0;
+				painter.drawText(point.x(), point.y(),
+						 Geonkick::doubleToStr(roundedValue, 1)
+						 + std::string("kHz ")
+						 + frequencyToNote(1000 * frequency));
+			}
+		}
         }
 }
 
@@ -302,7 +325,7 @@ void Envelope::drawLines(RkPainter &painter)
 	for (const auto& point : envelopePoints) {
                 auto scaledPoint = scaleUp(point);
 	        points.push_back(RkPoint(origin.x() + scaledPoint.x(),
-                                        origin.y() - scaledPoint.y()));
+					 origin.y() - scaledPoint.y()));
 	}
 
 	auto pen = painter.pen();
@@ -419,7 +442,7 @@ void Envelope::moveSelectedPoint(int x, int y)
 {
         if (!pointSelected || envelopePoints.empty())
 		return;
-
+	y = std::clamp(y, 0, H());
         auto scaledPoint = scaleDown(RkPoint(x, y));
         auto &selectedPoint = envelopePoints[selectedPointIndex];
 	if (scaledPoint.x() < getLeftPointLimit())
@@ -428,14 +451,7 @@ void Envelope::moveSelectedPoint(int x, int y)
                 selectedPoint.setX(getRightPointLimit());
 	else
                 selectedPoint.setX(scaledPoint.x());
-
-	if (y < 0)
-                selectedPoint.setY(0);
-	else if (y > H())
-                selectedPoint.setY(1);
-	else
-                selectedPoint.setY(scaledPoint.y());
-
+	selectedPoint.setY(scaledPoint.y());
         pointUpdatedEvent(selectedPointIndex, selectedPoint.x(), selectedPoint.y());
 }
 
@@ -499,7 +515,7 @@ void Envelope::setType(Type type)
 {
         if (isSupportedType(type)) {
                 envelopeType = type;
-                updatePoints();
+                updateEnvelope();
         }
 }
 
@@ -547,10 +563,24 @@ void Envelope::setDrawingArea(const RkRect &rect)
         drawingArea = rect;
 }
 
+void Envelope::updateEnvelope()
+{
+}
+
+void Envelope::updatePoints()
+{
+}
+
 RkRealPoint Envelope::scaleDown(const RkPoint &point)
 {
-        RkRealPoint scaledPoint;
+	if (applyType == ApplyType::Logarithmic) {
+		return {static_cast<double>(point.x()) / W(),
+			static_cast<double>(point.y()) / H()};
+	}
+
+	RkRealPoint scaledPoint;
         if (type() == Type::Amplitude
+	    || type() == Type::FilterQFactor
             || type() == Type::DistortionDrive
             || type() == Type::DistortionVolume
             || type() == Type::PitchShift) {
@@ -558,18 +588,22 @@ RkRealPoint Envelope::scaleDown(const RkPoint &point)
                                           static_cast<double>(point.y()) / H());
         } else {
                 scaledPoint.setX(static_cast<double>(point.x()) / W());
-                double logVal = (static_cast<double>(point.y()) / H()) * (log10(envelopeAmplitude()) - log10(20));
+                auto k = static_cast<double>(point.y()) / H();
+                double logVal = k * (log10(envelopeAmplitude()) - log10(20));
                 double val = pow(10, logVal + log10(20));
                 scaledPoint.setY(val / envelopeAmplitude());
         }
-
         return scaledPoint;
 }
 
 RkPoint Envelope::scaleUp(const RkRealPoint &point)
 {
-        int x, y;
+	if (getApplyType() == ApplyType::Logarithmic)
+		return RkPoint(point.x() * W(), point.y() * H());
+	
+	int x, y;
         if (type() == Type::Amplitude
+	    || type() == Type::FilterQFactor
             || type() == Type::DistortionDrive
             || type() == Type::DistortionVolume
             || type() == Type::PitchShift) {
@@ -648,6 +682,7 @@ std::string Envelope::frequencyToNote(rk_real f) const
 double Envelope::convertToHumanValue(double val) const
 {
         if (type() == Envelope::Type::Amplitude
+	    || type() == Type::FilterQFactor
             || type() == Type::DistortionDrive
             || type() == Type::DistortionVolume) {
                 val *= envelopeAmplitude();
@@ -659,7 +694,10 @@ double Envelope::convertToHumanValue(double val) const
                 val *= envelopeAmplitude();
                 return 2 * val - envelopeAmplitude();
         } else if (type() == Envelope::Type::Frequency || type() == Type::FilterCutOff) {
-                val *= envelopeAmplitude();
+                if (getApplyType() == ApplyType::Linear) 
+			val *= envelopeAmplitude();
+                else 
+	                return pow(10, (log10(envelopeAmplitude()) - log10(20)) * val + log10(20));
         }
         return val;
 }
@@ -670,6 +708,7 @@ double Envelope::convertFromHumanValue(double val) const
                 return val;
         
         if (type() == Envelope::Type::Amplitude
+	    || type() == Type::FilterQFactor
             || type() == Type::DistortionDrive
             || type() == Type::DistortionVolume) {
                 val /= envelopeAmplitude();
@@ -678,10 +717,18 @@ double Envelope::convertFromHumanValue(double val) const
         } else if (type() == Type::PitchShift) {
                 val = (val / envelopeAmplitude() + 1.0) / 2.0;
         } else if (type() == Envelope::Type::Frequency || type() == Type::FilterCutOff) {
-                val /= envelopeAmplitude();
+                if (getApplyType() == ApplyType::Linear) {
+                        val /= envelopeAmplitude();
+                } else {
+			if (val >= 20 && envelopeAmplitude() >= 20) {
+				val = log10(val / 20) / log10(envelopeAmplitude() / 20);
+				return std::clamp(val, 0.0, 1.0);
+			}
+                }
+                return 0;
         }
 
-        return std::clamp(val, 0.0, 1.0);        
+        return std::clamp(val, 0.0, 1.0);
 }
 
 bool Envelope::hasEditingPoint() const
@@ -709,3 +756,13 @@ std::string Envelope::getCurrentPointInfo() const
         info += ", " + Geonkick::doubleToStr(point.x() * envelopeLength(), 0) + "ms";
         return info;
 }
+
+void Envelope::setApplyType(Envelope::ApplyType apply)
+{
+	applyType = apply;
+}
+
+Envelope::ApplyType Envelope::getApplyType() const
+{
+	return applyType;
+}
