@@ -61,7 +61,7 @@ public:
      void createPointInfoLabel();
 
  private:
-     std::unordered_map<int, std::shared_ptr<Envelope>> envelopes;
+     std::unordered_map<int, std::unique_ptr<Envelope>> envelopes;
      EnvelopeWidgetDrawingArea *drawArea;
      GeonkickButton *layer1Button;
      GeonkickButton *layer2Button;
