@@ -31,10 +31,11 @@ class RkPaintEvent;
 class RkMouseEvent;
 class RkCloseEvent;
 class GeonkickButton;
-class RkLabel;
 class GeonkickSlider;
 class PathListModel;
 class RkList;
+class RkContainer;
+class GeonkickButton;
 
 class FilesView: public GeonkickWidget {
  public:
@@ -126,22 +127,27 @@ class FileDialog: public GeonkickWidget {
         AcceptStatus acceptStatus() const;
         void setFilters(const std::vector<std::string> &filters);
         void setHomeDirectory(const std::string &path);
+        bool createDirectory(const std::filesystem::path &dir);
+        void bookmarkDirectory(const std::filesystem::path &dir, bool bookmark = true);
+        bool isPathBookmarked(const std::filesystem::path &path) const;
 
  protected:
+        void createBookmarkDirectoryControls(RkContainer *container);
+        void createNewDirectoryControls(RkContainer *container);
+        void updateBookmarkButton(const std::filesystem::path &path);
         void onAccept();
         void onCancel();
-        void onPathChanged(const std::string &pathName);
         void closeEvent(RkCloseEvent *event) final;
 
  private:
         RkLineEdit *fileNameEdit;
         Type dialogType;
         FilesView *filesView;
-        RkLabel *pathLabel;
         std::string pathSelected;
         AcceptStatus status;
         PathListModel* shortcutDirectoriesModel;
         RkList *shortcutDirectoriesView;
+        GeonkickButton *bookmarkDirectoryButton;
 };
 
 #endif // GEONKICK_FILE_DIALOG_H
