@@ -67,6 +67,8 @@ double GeneralEnvelope::envelopeAmplitude(void) const
                 return geonkickApi->kickAmplitude();
         else if (type() == Envelope::Type::FilterCutOff)
                 return geonkickApi->kickFilterFrequency();
+	else if (type() == Envelope::Type::FilterQFactor)
+                return geonkickApi->kickFilterQFactor();
 	else if (type() == Envelope::Type::DistortionDrive)
 		return geonkickApi->getDistortionDrive() / (pow(10, 36.0 / 20));
         else if (type() == Envelope::Type::DistortionVolume)
@@ -75,6 +77,13 @@ double GeneralEnvelope::envelopeAmplitude(void) const
         return 0;
 }
 
+void GeneralEnvelope::updateEnvelope()
+{
+	auto applyType = geonkickApi->getKickEnvelopeApplyType(type());
+	setApplyType(applyType);
+	updatePoints();
+}
+
 void GeneralEnvelope::updatePoints()
 {
         setPoints(geonkickApi->getKickEnvelopePoints(type()));
