@@ -2,7 +2,7 @@
  * File name: geonkick_api.cpp
  * Project: Geonkick (A kick synthesizer)
  *
- * Copyright (C) 2017 Iurie Nistor 
+ * Copyright (C) 2017 Iurie Nistor
  *
  * This file is part of Geonkick.
  *
@@ -22,6 +22,7 @@
  */
 
 #include "geonkick_api.h"
+#include "DesktopPaths.h"
 #include "oscillator.h"
 #include "globals.h"
 #include "percussion_state.h"
@@ -29,6 +30,7 @@
 #include "preset.h"
 #include "preset_folder.h"
 #include "UiSettings.h"
+#include "GeonkickConfig.h"
 
 #include <RkEventQueue.h>
 
@@ -47,9 +49,13 @@ GeonkickApi::GeonkickApi(int sample_rate, InstanceType instance, geonkick *dsp)
         , clipboardPercussion{nullptr}
         , uiSettings{std::make_unique<UiSettings>()}
 	, sampleRate{sample_rate}
+        , scaleFactor{1.0}
 {
         setupPaths();
         uiSettings->setSamplesBrowserPath(getSettings("GEONKICK_CONFIG/HOME_PATH"));
+        GeonkickConfig cfg;
+        scaleFactor = cfg.getScaleFactor();
+        forceMidiChannel(cfg.getMidiChannel(), cfg.isMidiChannelForced());
 }
 
 GeonkickApi::~GeonkickApi()
@@ -77,17 +83,19 @@ void GeonkickApi::setEventQueue(RkEventQueue *queue)
 {
         std::lock_guard<std::mutex> lock(apiMutex);
         eventQueue = queue;
+        if (eventQueue)
+                eventQueue->setScaleFactor(getScaleFactor());
 }
 
 bool GeonkickApi::initDSP()
 {
-          if (!geonkickApi) {
-                  if (geonkick_create(&geonkickApi, sampleRate) != GEONKICK_OK) {
-	                  GEONKICK_LOG_ERROR("can't create geonkick API");
-                          return false;
-                  }
-	  }
-	  return true;
+        if (!geonkickApi) {
+                if (geonkick_create(&geonkickApi, sampleRate) != GEONKICK_OK) {
+                        GEONKICK_LOG_ERROR("can't create geonkick API");
+                        return false;
+                }
+        }
+        return true;
 }
 
 bool GeonkickApi::init()
@@ -96,13 +104,14 @@ bool GeonkickApi::init()
 	        return false;
 
 	loadPresets();
+
         jackEnabled = geonkick_is_module_enabed(geonkickApi, GEONKICK_MODULE_JACK);
 	geonkick_enable_synthesis(geonkickApi, false);
 
-	auto n = getPercussionsNumber();
+	auto nInstruments = numberOfInstruments();
         auto nChannels = numberOfChannels();
-        kickBuffers = std::vector<std::vector<gkick_real>>(n);
-	for (decltype(n) i = 0; i < n; i++) {
+        kickBuffers = std::vector<std::vector<gkick_real>>(nInstruments);
+	for (decltype(nInstruments) i = 0; i < nInstruments; i++) {
                 auto state = getDefaultPercussionState();
                 state->setId(i);
                 state->setChannel(i % nChannels);
@@ -119,19 +128,29 @@ bool GeonkickApi::init()
         return true;
 }
 
+size_t GeonkickApi::numberOfInstruments()
+{
+        return geonkick_instruments_number();
+}
+
 size_t GeonkickApi::numberOfChannels()
 {
         return geonkick_channels_number();
 }
 
+size_t GeonkickApi::numberOfMidiChannels()
+{
+        return geonkick_midi_channels_number();
+}
+
 std::unique_ptr<KitState> GeonkickApi::getDefaultKitState()
 {
         return std::make_unique<KitState>();
 }
 
-std::shared_ptr<PercussionState> GeonkickApi::getDefaultPercussionState()
+std::unique_ptr<PercussionState> GeonkickApi::getDefaultPercussionState()
 {
-        std::shared_ptr<PercussionState> state = std::make_shared<PercussionState>();
+        auto state = std::make_unique<PercussionState>();
         state->setName("Default");
         state->setId(0);
         state->setPlayingKey(-1);
@@ -148,7 +167,10 @@ std::shared_ptr<PercussionState> GeonkickApi::getDefaultPercussionState()
         envelope.push_back({0, 1});
         envelope.push_back({1, 1});
         state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::Amplitude, envelope);
-        state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterCutOff, envelope);
+        state->setKickEnvelopeApplyType(GeonkickApi::EnvelopeType::FilterCutOff,
+					EnvelopeApplyType::Logarithmic);
+	state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterCutOff, envelope);
+	state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterQFactor, envelope);
 	state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionDrive, envelope);
         state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionVolume, envelope);
         state->enableCompressor(false);
@@ -198,14 +220,26 @@ std::shared_ptr<PercussionState> GeonkickApi::getDefaultPercussionState()
                         state->setOscillatorFilterFactor(index, 10);
                         state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::Amplitude);
                         if (osc != GeonkickApi::OscillatorType::Noise) {
+				state->setOscillatorEnvelopeApplyType(index,
+								      GeonkickApi::EnvelopeType::Frequency,
+								      GeonkickApi::EnvelopeApplyType::Logarithmic);
                                 state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::Frequency);
                                 state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::PitchShift);
                                 std::vector<RkRealPoint> env = envelope;
                                 env[0].setY(0.5);
                                 env[1].setY(0.5);
                                 state->setOscillatorEnvelopePoints(index, env, GeonkickApi::EnvelopeType::PitchShift);
+
                         }
-                        state->setOscillatorEnvelopePoints(index, envelope, GeonkickApi::EnvelopeType::FilterCutOff);
+                        state->setOscillatorEnvelopePoints(index,
+							   envelope,
+							   GeonkickApi::EnvelopeType::FilterCutOff);
+			state->setOscillatorEnvelopePoints(index,
+							   envelope,
+							   GeonkickApi::EnvelopeType::FilterQFactor);
+			state->setOscillatorEnvelopeApplyType(index,
+							      GeonkickApi::EnvelopeType::FilterCutOff,
+							      GeonkickApi::EnvelopeApplyType::Logarithmic);
                         state->setOscillatorPitchShift(index, 12);
                 }
         }
@@ -213,7 +247,7 @@ std::shared_ptr<PercussionState> GeonkickApi::getDefaultPercussionState()
         return state;
 }
 
-void GeonkickApi::setPercussionState(const std::shared_ptr<PercussionState> &state)
+void GeonkickApi::setPercussionState(const std::unique_ptr<PercussionState> &state)
 {
         if (!state)
                 return;
@@ -225,6 +259,7 @@ void GeonkickApi::setPercussionState(const std::shared_ptr<PercussionState> &sta
         setPercussionName(state->getId(), state->getName());
         setPercussionPlayingKey(state->getId(), state->getPlayingKey());
         setPercussionChannel(state->getId(), state->getChannel());
+        setPercussionMidiChannel(state->getId(), state->getMidiChannel());
         mutePercussion(state->getId(), state->isMuted());
         soloPercussion(state->getId(), state->isSolo());
         for (auto i = 0; i < 3; i++) {
@@ -241,8 +276,12 @@ void GeonkickApi::setPercussionState(const std::shared_ptr<PercussionState> &sta
         setKickFilterType(state->getKickFilterType());
         setKickEnvelopePoints(GeonkickApi::EnvelopeType::Amplitude,
                               state->getKickEnvelopePoints(GeonkickApi::EnvelopeType::Amplitude));
-        setKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterCutOff,
+        setKickEnvelopeApplyType(GeonkickApi::EnvelopeType::FilterCutOff,
+				 state->getKickEnvelopeApplyType(GeonkickApi::EnvelopeType::FilterCutOff));
+	setKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterCutOff,
                               state->getKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterCutOff));
+	setKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterQFactor,
+                              state->getKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterQFactor));
 	setKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionDrive,
                               state->getKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionDrive));
         setKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionVolume,
@@ -276,7 +315,7 @@ void GeonkickApi::setPercussionState(const std::string &data)
         setPercussionState(state);
 }
 
-std::shared_ptr<PercussionState> GeonkickApi::getPercussionState(size_t id) const
+std::unique_ptr<PercussionState> GeonkickApi::getPercussionState(size_t id) const
 {
         if (id == currentPercussion()) {
                 return getPercussionState();
@@ -293,15 +332,16 @@ std::shared_ptr<PercussionState> GeonkickApi::getPercussionState(size_t id) cons
         }
 }
 
-std::shared_ptr<PercussionState> GeonkickApi::getPercussionState() const
+std::unique_ptr<PercussionState> GeonkickApi::getPercussionState() const
 {
-        auto state = std::make_shared<PercussionState>();
+        auto state = std::make_unique<PercussionState>();
         state->setId(currentPercussion());
         state->setName(getPercussionName(state->getId()));
         state->setLimiterValue(limiterValue());
         state->tuneOutput(isAudioOutputTuned(state->getId()));
         state->setPlayingKey(getPercussionPlayingKey(state->getId()));
         state->setChannel(getPercussionChannel(state->getId()));
+        state->setMidiChannel(getPercussionMidiChannel(state->getId()));
         state->setMute(isPercussionMuted(state->getId()));
         state->setSolo(isPercussionSolo(state->getId()));
         for (int i = 0; i < 3; i++) {
@@ -316,8 +356,12 @@ std::shared_ptr<PercussionState> GeonkickApi::getPercussionState() const
         state->setKickFilterType(kickFilterType());
         state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::Amplitude,
                                      getKickEnvelopePoints(GeonkickApi::EnvelopeType::Amplitude));
+	state->setKickEnvelopeApplyType(GeonkickApi::EnvelopeType::FilterCutOff,
+                                     getKickEnvelopeApplyType(GeonkickApi::EnvelopeType::FilterCutOff));
         state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterCutOff,
                                      getKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterCutOff));
+	state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterQFactor,
+                                     getKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterQFactor));
 	state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionDrive,
                                      getKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionDrive));
         state->setKickEnvelopePoints(GeonkickApi::EnvelopeType::DistortionVolume,
@@ -346,7 +390,7 @@ std::shared_ptr<PercussionState> GeonkickApi::getPercussionState() const
 
 void GeonkickApi::getOscillatorState(GeonkickApi::Layer layer,
                                      OscillatorType osc,
-                                     const std::shared_ptr<PercussionState> &state) const
+                                     const std::unique_ptr<PercussionState> &state) const
 {
         auto temp = currentLayer;
         currentLayer = layer;
@@ -369,20 +413,26 @@ void GeonkickApi::getOscillatorState(GeonkickApi::Layer layer,
         auto points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::Amplitude);
         state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::Amplitude);
         if (osc != OscillatorType::Noise) {
-                points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::Frequency);
+		auto applyType = getOscillatorEnvelopeApplyType(index, GeonkickApi::EnvelopeType::Frequency);
+	        state->setOscillatorEnvelopeApplyType(index, GeonkickApi::EnvelopeType::Frequency, applyType);
+		points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::Frequency);
                 state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::Frequency);
                 points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::PitchShift);
                 state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::PitchShift);
         }
+        auto applyType = getOscillatorEnvelopeApplyType(index, GeonkickApi::EnvelopeType::FilterCutOff);
+        state->setOscillatorEnvelopeApplyType(index, GeonkickApi::EnvelopeType::FilterCutOff, applyType);
         points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::FilterCutOff);
         state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::FilterCutOff);
+	points = oscillatorEvelopePoints(index, GeonkickApi::EnvelopeType::FilterQFactor);
+        state->setOscillatorEnvelopePoints(index, points, GeonkickApi::EnvelopeType::FilterQFactor);
         state->setOscillatorAsFm(index, isOscillatorAsFm(index));
         currentLayer = temp;
 }
 
 void GeonkickApi::setOscillatorState(GeonkickApi::Layer layer,
                                      OscillatorType oscillator,
-                                     const std::shared_ptr<PercussionState> &state)
+                                     const std::unique_ptr<PercussionState> &state)
 {
         auto temp = currentLayer;
         currentLayer = layer;
@@ -407,6 +457,9 @@ void GeonkickApi::setOscillatorState(GeonkickApi::Layer layer,
         setOscillatorEvelopePoints(osc, EnvelopeType::Amplitude,
                                    state->oscillatorEnvelopePoints(osc, EnvelopeType::Amplitude));
         if (oscillator != OscillatorType::Noise) {
+		setOscillatorEnvelopeApplyType(osc, EnvelopeType::Frequency,
+					       state->getOscillatorEnvelopeApplyType(osc,
+										     EnvelopeType::Frequency));
                 setOscillatorEvelopePoints(osc, EnvelopeType::Frequency,
                                            state->oscillatorEnvelopePoints(osc,
                                            EnvelopeType::Frequency));
@@ -414,10 +467,14 @@ void GeonkickApi::setOscillatorState(GeonkickApi::Layer layer,
                                            state->oscillatorEnvelopePoints(osc,
                                            EnvelopeType::PitchShift));
         }
+	setOscillatorEnvelopeApplyType(osc, EnvelopeType::FilterCutOff,
+				       state->getOscillatorEnvelopeApplyType(osc,
+									     EnvelopeType::FilterCutOff));
         setOscillatorEvelopePoints(osc, EnvelopeType::FilterCutOff,
                                    state->oscillatorEnvelopePoints(osc, EnvelopeType::FilterCutOff));
+	setOscillatorEvelopePoints(osc, EnvelopeType::FilterQFactor,
+                                   state->oscillatorEnvelopePoints(osc, EnvelopeType::FilterQFactor));
         setOscillatorAsFm(osc, state->isOscillatorAsFm(osc));
-
         currentLayer = temp;
 }
 
@@ -431,7 +488,7 @@ std::unique_ptr<KitState> GeonkickApi::getKitState() const
         for (const auto &id : ordredPercussionIds()) {
                 auto state = getPercussionState(id);
                 state->setId(i);
-                kit->addPercussion(state);
+                kit->addPercussion(std::move(state));
                 GEONKICK_LOG_DEBUG("PER: " << state->getName() << ": id = " << state->getId());
                 i++;
         }
@@ -447,7 +504,7 @@ bool GeonkickApi::setKitState(const std::string &data)
 
 bool GeonkickApi::setKitState(const std::unique_ptr<KitState> &state)
 {
-        auto n = getPercussionsNumber();
+        auto n = numberOfInstruments();
         for (decltype(n) i = 0; i < n; i++)
                 enablePercussion(i, false);
         setKitName(state->getName());
@@ -493,6 +550,17 @@ std::vector<RkRealPoint> GeonkickApi::oscillatorEvelopePoints(int oscillatorInde
         return points;
 }
 
+GeonkickApi::EnvelopeApplyType GeonkickApi::getOscillatorEnvelopeApplyType(int index,
+									   EnvelopeType envelope) const
+{
+	enum gkick_envelope_apply_type applyType;
+	geonkick_osc_envelope_get_apply_type(geonkickApi,
+					     static_cast<size_t>(getOscIndex(index)),
+					     static_cast<size_t>(envelope),
+					     &applyType);
+	return static_cast<EnvelopeApplyType>(applyType);
+}
+
 void GeonkickApi::setOscillatorEvelopePoints(int index,
                                              EnvelopeType envelope,
                                              const std::vector<RkRealPoint> &points)
@@ -514,6 +582,16 @@ void GeonkickApi::setOscillatorEvelopePoints(int index,
                                          points.size());
 }
 
+void GeonkickApi::setOscillatorEnvelopeApplyType(int index,
+						 EnvelopeType envelope,
+						 EnvelopeApplyType applyType)
+{
+	geonkick_osc_envelope_set_apply_type(geonkickApi,
+					     getOscIndex(index),
+					     static_cast<int>(envelope),
+					     static_cast<enum gkick_envelope_apply_type>(applyType));
+}
+
 void GeonkickApi::addOscillatorEnvelopePoint(int oscillatorIndex,
                                              EnvelopeType envelope,
                                              const RkRealPoint &point)
@@ -636,6 +714,15 @@ std::vector<RkRealPoint> GeonkickApi::getKickEnvelopePoints(EnvelopeType envelop
         return points;
 }
 
+GeonkickApi::EnvelopeApplyType GeonkickApi::getKickEnvelopeApplyType(EnvelopeType envelope) const
+{
+	enum gkick_envelope_apply_type applyType;
+	geonkick_kick_env_get_apply_type(geonkickApi,
+					 static_cast<enum geonkick_envelope_type>(envelope),
+					 &applyType);
+	return static_cast<EnvelopeApplyType>(applyType);
+}
+
 void GeonkickApi::setKickEnvelopePoints(EnvelopeType envelope,
                                         const std::vector<RkRealPoint> &points)
 {
@@ -652,6 +739,13 @@ void GeonkickApi::setKickEnvelopePoints(EnvelopeType envelope,
                                           points.size());
 }
 
+void GeonkickApi::setKickEnvelopeApplyType(EnvelopeType envelope, EnvelopeApplyType applyType)
+{
+        geonkick_kick_env_set_apply_type(geonkickApi,
+					 static_cast<enum geonkick_envelope_type>(envelope),
+					 static_cast<enum gkick_envelope_apply_type>(applyType));
+}
+
 void GeonkickApi::enableKickFilter(bool b)
 {
         geonkick_kick_filter_enable(geonkickApi, b);
@@ -906,6 +1000,7 @@ void GeonkickApi::kickUpdatedCallback(void *arg,
                                       size_t size,
                                       size_t id)
 {
+        GEONKICK_LOG_DEBUG("size[" << id << "] : " << size);
         std::vector<gkick_real> buffer(size, 0);
         std::memcpy(buffer.data(), buff, size * sizeof(gkick_real));
         GeonkickApi *obj = static_cast<GeonkickApi*>(arg);
@@ -938,13 +1033,17 @@ double GeonkickApi::getLimiterLevelerValue(size_t index) const
 void GeonkickApi::updateKickBuffer(const std::vector<gkick_real> &&buffer,
                                    size_t id)
 {
+        GEONKICK_LOG_DEBUG("id: " << id);
         std::lock_guard<std::mutex> lock(apiMutex);
-        if (id < getPercussionsNumber())
+        if (id < numberOfInstruments()) {
+                GEONKICK_LOG_DEBUG("kickBuffers[id] = buffer" << id);
                 kickBuffers[id] = buffer;
+        }
         if (eventQueue && id == currentPercussion()) {
                 auto act = std::make_unique<RkAction>();
                 act->setCallback([&](void){ kickUpdated(); });
                 eventQueue->postAction(std::move(act));
+                GEONKICK_LOG_DEBUG("eventQueue->postAction / kickUpdated()");
         }
 }
 
@@ -955,7 +1054,7 @@ std::vector<gkick_real> GeonkickApi::getInstrumentBuffer(int id) const
                 if (static_cast<decltype(kickBuffers.size())>(id) < kickBuffers.size())
                         return kickBuffers[id];
         }
-                return std::vector<gkick_real>();
+        return std::vector<gkick_real>();
 }
 
 std::vector<gkick_real> GeonkickApi::getKickBuffer() const
@@ -991,14 +1090,6 @@ void GeonkickApi::playSamplePreview()
         geonkick_play_sample_preview(geonkickApi);
 }
 
-// This function is called only from the audio thread.
-gkick_real GeonkickApi::getAudioFrame(int channel) const
-{
-        gkick_real val;
-        geonkick_get_audio_frame(geonkickApi, channel, &val);
-        return val;
-}
-
 void GeonkickApi::process(float** out, size_t offset, size_t size)
 {
         geonkick_audio_process(geonkickApi, out, offset, size);
@@ -1269,11 +1360,6 @@ bool GeonkickApi::isPercussionSolo(size_t id) const
         return solo;
 }
 
-size_t GeonkickApi::getPercussionsNumber() const
-{
-	return geonkick_percussion_number();
-}
-
 int GeonkickApi::getUnusedPercussion() const
 {
         int index;
@@ -1296,7 +1382,7 @@ bool GeonkickApi::isPercussionEnabled(int index) const
 
 size_t GeonkickApi::enabledPercussions() const
 {
-        auto n = getPercussionsNumber();
+        auto n = numberOfInstruments();
         size_t enabled  = 0;
         for (decltype(n) i = 0; i < n; i++) {
                 if (isPercussionEnabled(i))
@@ -1348,6 +1434,38 @@ int GeonkickApi::getPercussionChannel(int index) const
         return channel;
 }
 
+bool GeonkickApi::setPercussionMidiChannel(int index, size_t channel)
+{
+        auto res = geonkick_set_midi_channel(geonkickApi,
+                                             index,
+                                             channel);
+        return res == GEONKICK_OK;
+}
+
+int GeonkickApi::getPercussionMidiChannel(int index) const
+{
+        signed char channel;
+        auto res = geonkick_get_midi_channel(geonkickApi,
+                                             index,
+                                             &channel);
+        if (res != GEONKICK_OK)
+                return -1;
+        return channel;
+}
+
+bool GeonkickApi::forceMidiChannel(size_t channel, bool force)
+{
+        auto res = geonkick_force_midi_channel(geonkickApi, channel, force);
+        return res == GEONKICK_OK;
+}
+
+bool GeonkickApi::isMidiChannelForced() const
+{
+        bool forced = false;
+        geonkick_ged_forced_midi_channel(geonkickApi, nullptr, &forced);
+        return forced;
+}
+
 bool GeonkickApi::setPercussionName(int index, const std::string &name)
 {
         auto res = geonkick_set_percussion_name(geonkickApi,
@@ -1359,7 +1477,7 @@ bool GeonkickApi::setPercussionName(int index, const std::string &name)
 
 std::string GeonkickApi::getPercussionName(int index) const
 {
-        auto n = getPercussionsNumber();
+        auto n = numberOfInstruments();
         if (index > -1 && index < static_cast<decltype(index)>(n)) {
                 char name[30];
                 geonkick_get_percussion_name(geonkickApi,
@@ -1554,12 +1672,13 @@ void GeonkickApi::copyToClipboard()
 void GeonkickApi::pasteFromClipboard()
 {
         if (clipboardPercussion) {
-                auto state = std::make_shared<PercussionState>(*clipboardPercussion);
+                auto state = std::make_unique<PercussionState>(*clipboardPercussion);
                 auto currId = currentPercussion();
                 state->setId(currId);
                 state->setName(getPercussionName(currId));
                 state->setPlayingKey(getPercussionPlayingKey(currId));
                 state->setChannel(getPercussionChannel(currId));
+                state->setMidiChannel(getPercussionMidiChannel(currId));
                 state->setMute(isPercussionMuted(currId));
                 state->setSolo(isPercussionSolo(currId));
                 setPercussionState(state);
@@ -1668,21 +1787,21 @@ void GeonkickApi::loadPresets()
         std::unordered_set<std::string> prestsPaths;
         auto presetsPathSufix = std::filesystem::path(GEONKICK_APP_NAME) / "presets";
         std::filesystem::path presetsPath = getSettings("GEONKICK_CONFIG/USER_PRESETS_PATH");
-        prestsPaths.insert(presetsPath);
+        prestsPaths.insert(presetsPath.string());
 
 #ifdef GEONKICK_DATA_DIR
-        prestsPaths.insert(std::filesystem::path(GEONKICK_DATA_DIR) / presetsPathSufix);
+        prestsPaths.insert((std::filesystem::path(GEONKICK_DATA_DIR) / presetsPathSufix).string());
 #endif // GEONKICK_DATA_DIR
 
         const char *dataDirs = std::getenv("XDG_DATA_DIRS");
         if (dataDirs == nullptr || *dataDirs == '\0') {
-                prestsPaths.insert("/usr/share" / presetsPathSufix);
-                prestsPaths.insert("/usr/local/share" / presetsPathSufix);
+                prestsPaths.insert((std::filesystem::path("/usr/share") / presetsPathSufix).string());
+                prestsPaths.insert((std::filesystem::path("/usr/local/share") / presetsPathSufix).string());
         } else {
                 std::stringstream ss(dataDirs);
                 std::string path;
                 while (std::getline(ss, path, ':'))
-                        prestsPaths.insert(std::filesystem::path(path) / presetsPathSufix);
+                        prestsPaths.insert((std::filesystem::path(path) / presetsPathSufix).string());
         }
 
         for (const auto &path: prestsPaths) {
@@ -1693,6 +1812,14 @@ void GeonkickApi::loadPresets()
                         GEONKICK_LOG_ERROR("error on reading path: " << path << ": " << e.what());
                 }
         }
+
+        // Load custom preset folders.
+        GeonkickConfig cfg;
+        for (const auto &folder: cfg.getCustomPresetFolders()) {
+                auto presetFolder = std::make_unique<PresetFolder>(folder);
+                presetFolder->setAsCustom();
+                presetsFoldersList.emplace_back(std::move(presetFolder));
+        }
 }
 
 void GeonkickApi::loadPresetsFolders(const std::filesystem::path &path)
@@ -1701,12 +1828,8 @@ void GeonkickApi::loadPresetsFolders(const std::filesystem::path &path)
                 for (const auto &entry : std::filesystem::directory_iterator(path)) {
                         if (!entry.path().empty() && std::filesystem::is_directory(entry.path())) {
                                 auto presetFolder = std::make_unique<PresetFolder>(entry.path());
-                                GEONKICK_LOG_DEBUG("preset folder " << presetFolder->path());
-                                if (!presetFolder->loadPresets()) {
-                                        GEONKICK_LOG_ERROR("can't load preset from folder " << presetFolder->path());
-                                } else if (presetFolder->numberOfPresets() > 0) {
+                                if (presetFolder->numberOfPresets() > 0)
                                         presetsFoldersList.push_back(std::move(presetFolder));
-                                }
                         }
                 }
         } catch(...) {
@@ -1716,40 +1839,26 @@ void GeonkickApi::loadPresetsFolders(const std::filesystem::path &path)
 
 void GeonkickApi::setupPaths()
 {
-        std::filesystem::path dataPath;
-        const char *dataHome = std::getenv("XDG_DATA_HOME");
-        if (dataHome == nullptr || *dataHome == '\0') {
-                const char *homeDir = std::getenv("HOME");
-                if (homeDir == nullptr || *homeDir == '\0') {
-                        GEONKICK_LOG_ERROR("can't get home directory");
-                        return;
-                }
-                dataPath = homeDir / std::filesystem::path(".local/share")
-                        / std::filesystem::path(GEONKICK_APP_NAME);
-                setSettings("GEONKICK_CONFIG/HOME_PATH", homeDir);
-        } else {
-                dataPath = dataHome / std::filesystem::path(GEONKICK_APP_NAME);
-                setSettings("GEONKICK_CONFIG/HOME_PATH", dataHome);
-        }
+	DesktopPaths desktopPaths;
+	setSettings("GEONKICK_CONFIG/HOME_PATH", desktopPaths.getHomePath().string());
+	setSettings("GEONKICK_CONFIG/USER_PRESETS_PATH", desktopPaths.getPresetsPath().string());
+	setSettings("GEONKICK_CONFIG/USER_DATA_PATH", desktopPaths.getDataPath().string());
 
-        try {
-                if (!std::filesystem::exists(dataPath)) {
-                        if (!std::filesystem::create_directories(dataPath)) {
-                                GEONKICK_LOG_ERROR("can't create path " << dataPath);
+	try {
+                if (!std::filesystem::exists(desktopPaths.getDataPath())) {
+                        if (!std::filesystem::create_directories(desktopPaths.getDataPath())) {
+                                GEONKICK_LOG_ERROR("can't create path " << desktopPaths.getDataPath());
                                 return;
                         }
                 }
-                setSettings("GEONKICK_CONFIG/USER_DATA_PATH", dataPath);
 
-                auto presetsPath = dataPath / std::filesystem::path("presets");
-                if (!std::filesystem::exists(presetsPath)) {
-                        if (!std::filesystem::create_directories(presetsPath)) {
-                                GEONKICK_LOG_ERROR("can't create path " << presetsPath);
+                if (!std::filesystem::exists(desktopPaths.getPresetsPath())) {
+                        if (!std::filesystem::create_directories(desktopPaths.getPresetsPath())) {
+                                GEONKICK_LOG_ERROR("can't create path " << desktopPaths.getPresetsPath());
                                 return;
                         }
                 }
-                setSettings("GEONKICK_CONFIG/USER_PRESETS_PATH", presetsPath);
-        } catch(const std::exception& e) {
+	} catch(const std::exception& e) {
                 GEONKICK_LOG_ERROR("error on setup user data paths: " << e.what());
         }
 }
@@ -1761,6 +1870,39 @@ PresetFolder* GeonkickApi::getPresetFolder(size_t index) const
         return nullptr;
 }
 
+PresetFolder* GeonkickApi::addPresetFolder(const std::filesystem::path &folder, bool custom)
+{
+        auto it = std::find_if(presetsFoldersList.cbegin(),
+                               presetsFoldersList.cend(), [&folder](const auto &e)
+                               {
+                                       return e->path() == folder;
+                               });
+        if (it == presetsFoldersList.cend()) {
+                auto presetFolder = std::make_unique<PresetFolder>(folder);
+                presetFolder->setAsCustom(custom);
+                GeonkickConfig cfg;
+                cfg.addCustomPresetFolder(presetFolder->path());
+                cfg.save();
+                return presetsFoldersList.emplace_back(std::move(presetFolder)).get();
+        }
+        return nullptr;
+}
+
+bool GeonkickApi::removePresetFolder(const PresetFolder *folder)
+{
+        auto folderPath = folder->path();
+        presetsFoldersList.erase(std::remove_if(presetsFoldersList.begin(),
+                                                presetsFoldersList.end(),
+                                                [&folder](const auto &p)
+                                                { return p->path() == folder->path(); }),
+                                 presetsFoldersList.end());
+
+        GeonkickConfig cfg;
+        cfg.removeCustomPresetFolder(folderPath);
+        cfg.save();
+        return true;
+}
+
 size_t GeonkickApi::numberOfPresetFolders() const
 {
         return presetsFoldersList.size();
@@ -1826,3 +1968,16 @@ double GeonkickApi::samplePreviewLimiter() const
         geonkick_get_sample_preview_limiter(geonkickApi, &val);
         return val;
 }
+
+double GeonkickApi::getScaleFactor() const
+{
+        return scaleFactor;
+}
+
+void GeonkickApi::setScaleFactor(double factor)
+{
+        scaleFactor = factor;
+        GeonkickConfig config;
+        config.setScaleFactor(scaleFactor);
+        config.save();
+}
