@@ -2,7 +2,7 @@
  * File name: kit_widget.h
  * Project: Geonkick (A percussion synthesizer)
  *
- * Copyright (C) 2020 Iurie Nistor <http://iuriepage.wordpress.com>
+ * Copyright (C) 2020 Iurie Nistor 
  *
  * This file is part of Geonkick.
  *
@@ -53,7 +53,7 @@ class KitWidget: public GeonkickWidget
         void showFileDialog(FileDialog::Type type);
         void openKit(const std::string &file);
         void saveKit(const std::string &file);
-        void keyPressEvent(RkKeyEvent *event);
+        void keyPressEvent(RkKeyEvent *event) override;
         void onUpdateLevelers();
         void exportKitDialog();
 
