@@ -41,6 +41,13 @@ void OscillatorEnvelope::updatePoints()
         setPoints(oscillator->envelopePoints(static_cast<Oscillator::EnvelopeType>(type())));
 }
 
+void OscillatorEnvelope::updateEnvelope()
+{
+	auto applyType = oscillator->envelopeApplyType(static_cast<Oscillator::EnvelopeType>(type()));
+	setApplyType(applyType);
+	updatePoints();
+}
+
 void OscillatorEnvelope::pointAddedEvent(double x, double y)
 {
         oscillator->addEnvelopePoint(static_cast<Oscillator::EnvelopeType>(type()), x, y);
@@ -54,7 +61,6 @@ void OscillatorEnvelope::pointUpdatedEvent(unsigned int index, double x, double
 void OscillatorEnvelope::pointRemovedEvent(unsigned int index)
 {
         oscillator->removeEnvelopePoint(static_cast<Oscillator::EnvelopeType>(type()), index);
-
 }
 
 double OscillatorEnvelope::envelopeLength(void) const
@@ -73,6 +79,8 @@ double OscillatorEnvelope::envelopeAmplitude() const
                 return oscillator->pitchShift();
         case Type::FilterCutOff:
                 return oscillator->filterFrequency();
+	case Type::FilterQFactor:
+                return oscillator->filterQFactor();
         default:
                 return 0;
         }
