@@ -32,6 +32,7 @@ PercussionState::PercussionState()
         , kickName{"Default"}
         , playingKey{-1}
         , outputChannel{0}
+        , outputMidiChannel{GeonkickTypes::geonkickAnyMidiChannel}
         , kickEnabled{true}
         , percussionMuted{false}
         , percussionSolo{false}
@@ -42,6 +43,7 @@ PercussionState::PercussionState()
         , kickFilterFrequency{200}
         , kickFilterQFactor{1.0}
         , kickFilterType{GeonkickApi::FilterType::LowPass}
+	, kickCutOffEnvelopeApplyType{GeonkickApi::EnvelopeApplyType::Logarithmic}
         , compressor{false, 0, 0, 0, 0, 0, 0}
         , distortion{false, 1.0, 1.0, 1.0}
         , layers{false, false, false}
@@ -55,7 +57,7 @@ PercussionState::PercussionState()
 bool PercussionState::loadFile(const std::string &file)
 {
         if (file.size() < 7) {
-                RK_LOG_ERROR("can't open preset.");
+                GEONKICK_LOG_ERROR("can't open preset.");
                 return false;
         }
 
@@ -63,14 +65,14 @@ bool PercussionState::loadFile(const std::string &file)
         if (filePath.extension().empty()
             || (filePath.extension() != ".gkick"
             && filePath.extension() != ".GKICK")) {
-                RK_LOG_ERROR("can't open preset. Wrong file format.");
+                GEONKICK_LOG_ERROR("can't open preset. Wrong file format.");
                 return false;
         }
 
         std::ifstream sfile;
         sfile.open(std::filesystem::absolute(filePath));
         if (!sfile.is_open()) {
-	        RK_LOG_ERROR("can't open preset file " << file);
+	        GEONKICK_LOG_ERROR("can't open preset file " << file);
                 return false;
         }
 
@@ -94,10 +96,10 @@ bool PercussionState::loadData(const std::string &data)
 	return true;
 }
 
-void PercussionState::loadObject(const rapidjson::Value &obj)
+bool PercussionState::loadObject(const rapidjson::Value &obj)
 {
         if (!obj.IsObject())
-                return;
+                return false;
 
         bool kickParsed = false;
         for (const auto &m: obj.GetObject()) {
@@ -109,7 +111,7 @@ void PercussionState::loadObject(const rapidjson::Value &obj)
         }
 
         if (!kickParsed)
-                return;
+                return false;
 
         for (const auto &m: obj.GetObject()) {
                 for (decltype(layers.size()) i = 0; i < layers.size(); i++) {
@@ -126,6 +128,7 @@ void PercussionState::loadObject(const rapidjson::Value &obj)
                         }
                 }
         }
+        return true;
 }
 
 size_t PercussionState::getId() const
@@ -163,6 +166,11 @@ void PercussionState::setChannel(size_t channel)
         outputChannel = channel;
 }
 
+void PercussionState::setMidiChannel(signed char channel)
+{
+        outputMidiChannel = channel;
+}
+
 void PercussionState::setMute(bool b)
 {
         percussionMuted = b;
@@ -188,6 +196,11 @@ size_t PercussionState::getChannel() const
         return outputChannel;
 }
 
+signed char PercussionState::getMidiChannel() const
+{
+        return outputMidiChannel;
+}
+
 bool PercussionState::isEnabled() const
 {
         return kickEnabled;
@@ -202,14 +215,14 @@ void PercussionState::initOscillators()
 {
         for (decltype(layers.size()) i = 0; i < layers.size(); i++) {
                 oscillators.insert({static_cast<int>(GeonkickApi::OscillatorType::Oscillator1)
-                                        + GKICK_OSC_GROUP_SIZE * i,
-                                        std::make_shared<OscillatorInfo>()});
+                                + GKICK_OSC_GROUP_SIZE * i,
+                                OscillatorInfo()});
                 oscillators.insert({static_cast<int>(GeonkickApi::OscillatorType::Oscillator2)
-                                        + GKICK_OSC_GROUP_SIZE * i,
-                                        std::make_shared<OscillatorInfo>()});
+                                + GKICK_OSC_GROUP_SIZE * i,
+                                OscillatorInfo()});
                 oscillators.insert({static_cast<int>(GeonkickApi::OscillatorType::Noise)
-                                        + GKICK_OSC_GROUP_SIZE * i,
-                                        std::make_shared<OscillatorInfo>()});
+                                + GKICK_OSC_GROUP_SIZE * i,
+                                OscillatorInfo()});
         }
 }
 
@@ -227,6 +240,8 @@ void PercussionState::parseKickObject(const rapidjson::Value &kick)
                         setId(m.value.GetInt());
                 if (m.name == "channel" && m.value.IsInt())
                         setChannel(m.value.GetInt());
+                if (m.name == "midiChannel" && m.value.IsInt())
+                        setMidiChannel(m.value.GetInt());
                 if (m.name == "mute" && m.value.IsBool())
                         setMute(m.value.GetBool());
                 if (m.name == "solo" && m.value.IsBool())
@@ -270,11 +285,14 @@ void PercussionState::parseKickObject(const rapidjson::Value &kick)
                 }
 
                 if (m.name == "filter" && m.value.IsObject()) {
+			auto applyType = GeonkickApi::EnvelopeApplyType::Linear;
                         for (const auto &el: m.value.GetObject()) {
                                 if (el.name == "enabled" && el.value.IsBool())
                                         enableKickFilter(el.value.GetBool());
                                 if (el.name == "cutoff" && el.value.IsDouble())
                                         setKickFilterFrequency(el.value.GetDouble());
+				if (el.name == "apply_type")
+					applyType = getApplyTypeFromObj(el.value);
                                 if (el.name == "factor" && el.value.IsDouble())
                                         setKickFilterQFactor(el.value.GetDouble());
                                 if (el.name == "type" && el.value.IsInt())
@@ -283,7 +301,12 @@ void PercussionState::parseKickObject(const rapidjson::Value &kick)
                                         setKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterCutOff,
                                                               parseEnvelopeArray(el.value));
                                 }
+				if (el.name == "qfactor_env" && el.value.IsArray()) {
+                                        setKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterQFactor,
+                                                              parseEnvelopeArray(el.value));
+                                }
                         }
+			setKickEnvelopeApplyType(GeonkickApi::EnvelopeType::FilterCutOff, applyType);
                 }
 
                 if (m.name == "compressor" && m.value.IsObject()) {
@@ -358,13 +381,19 @@ void PercussionState::parseOscillatorObject(int index,  const rapidjson::Value &
 
                 if (static_cast<GeonkickApi::OscillatorType>(index) != GeonkickApi::OscillatorType::Noise) {
                         if (m.name == "freq_env" && m.value.IsObject()) {
+				auto applyType = GeonkickApi::EnvelopeApplyType::Linear;
                                 for (const auto &el: m.value.GetObject()) {
                                         if (el.name == "amplitude" && el.value.IsDouble())
                                                 setOscillatorFrequency(index, el.value.GetDouble());
+					if (el.name == "apply_type")
+						applyType = getApplyTypeFromObj(el.value);
                                         if (el.name == "points" && el.value.IsArray())
                                                 setOscillatorEnvelopePoints(index, parseEnvelopeArray(el.value),
                                                                             GeonkickApi::EnvelopeType::Frequency);
                                 }
+			        setOscillatorEnvelopeApplyType(index,
+							       GeonkickApi::EnvelopeType::Frequency,
+							       applyType);
                         }
 
                         if (m.name == "pitchshift_env" && m.value.IsObject()) {
@@ -379,6 +408,7 @@ void PercussionState::parseOscillatorObject(int index,  const rapidjson::Value &
                 }
 
                 if (m.name == "filter" && m.value.IsObject()) {
+			auto applyType = GeonkickApi::EnvelopeApplyType::Linear;
                         for (const auto &el: m.value.GetObject()) {
                                 if (el.name == "enabled" && el.value.IsBool())
                                         setOscillatorFilterEnabled(index, el.value.GetBool());
@@ -386,14 +416,22 @@ void PercussionState::parseOscillatorObject(int index,  const rapidjson::Value &
                                         setOscillatorFilterCutOffFreq(index, el.value.GetDouble());
                                 if (el.name == "factor" && el.value.IsDouble())
                                         setOscillatorFilterFactor(index, el.value.GetDouble());
+				if (el.name == "apply_type")
+					applyType = getApplyTypeFromObj(el.value);
                                 if (el.name == "type" && el.value.IsInt())
                                         setOscillatorFilterType(index, static_cast<GeonkickApi::FilterType>(el.value.GetInt()));
                                 if (el.name == "cutoff_env" && el.value.IsArray()) {
                                         setOscillatorEnvelopePoints(index, parseEnvelopeArray(el.value),
                                                                     GeonkickApi::EnvelopeType::FilterCutOff);
                                 }
-
+				if (el.name == "qfactor_env" && el.value.IsArray()) {
+                                        setOscillatorEnvelopePoints(index, parseEnvelopeArray(el.value),
+                                                                    GeonkickApi::EnvelopeType::FilterQFactor);
+                                }
                         }
+			setOscillatorEnvelopeApplyType(index,
+						       GeonkickApi::EnvelopeType::FilterCutOff,
+						       applyType);
                 }
         }
 }
@@ -447,14 +485,42 @@ void PercussionState::setKickFilterType(GeonkickApi::FilterType type)
 void PercussionState::setKickEnvelopePoints(GeonkickApi::EnvelopeType envelope,
                                             const std::vector<RkRealPoint> &points)
 {
-        if (envelope == GeonkickApi::EnvelopeType::Amplitude)
+	switch(envelope) {
+	case GeonkickApi::EnvelopeType::Amplitude:
                 kickEnvelopePoints = points;
-        else if (envelope == GeonkickApi::EnvelopeType::FilterCutOff)
+		break;
+	case GeonkickApi::EnvelopeType::FilterCutOff:
                 kickFilterCutOffEnvelope = points;
-	else if (envelope == GeonkickApi::EnvelopeType::DistortionDrive)
+		break;
+	case GeonkickApi::EnvelopeType::FilterQFactor:
+                kickFilterQFactorEnvelope = points;
+		break;
+	case GeonkickApi::EnvelopeType::DistortionDrive:
 		kickDistortionDriveEnvelope = points;
-        else if (envelope == GeonkickApi::EnvelopeType::DistortionVolume)
+		break;
+	case GeonkickApi::EnvelopeType::DistortionVolume:
 		kickDistortionVolumeEnvelope = points;
+		break;
+	default:
+		break;
+	}
+}
+
+void PercussionState::setKickEnvelopeApplyType(GeonkickApi::EnvelopeType envelope,
+					       GeonkickApi::EnvelopeApplyType applyType)
+{
+
+	if (envelope == GeonkickApi::EnvelopeType::FilterCutOff)
+		kickCutOffEnvelopeApplyType = applyType;
+}
+
+GeonkickApi::EnvelopeApplyType
+PercussionState::getKickEnvelopeApplyType(GeonkickApi::EnvelopeType envelope) const
+{
+
+	if (envelope == GeonkickApi::EnvelopeType::FilterCutOff)
+		return kickCutOffEnvelopeApplyType;
+	return GeonkickApi::EnvelopeApplyType::Linear;
 }
 
 double PercussionState::getLimiterValue() const
@@ -495,25 +561,40 @@ GeonkickApi::FilterType PercussionState::getKickFilterType() const
 std::vector<RkRealPoint>
 PercussionState::getKickEnvelopePoints(GeonkickApi::EnvelopeType envelope) const
 {
-        if (envelope == GeonkickApi::EnvelopeType::Amplitude)
+	switch(envelope) {
+        case GeonkickApi::EnvelopeType::Amplitude:
                 return kickEnvelopePoints;
-        else if (envelope == GeonkickApi::EnvelopeType::FilterCutOff)
+        case GeonkickApi::EnvelopeType::FilterCutOff:
                 return kickFilterCutOffEnvelope;
-	else if (envelope == GeonkickApi::EnvelopeType::DistortionDrive)
+	case GeonkickApi::EnvelopeType::FilterQFactor:
+		return kickFilterQFactorEnvelope;
+	case GeonkickApi::EnvelopeType::DistortionDrive:
 		return kickDistortionDriveEnvelope;
-        else if (envelope == GeonkickApi::EnvelopeType::DistortionVolume)
+	case GeonkickApi::EnvelopeType::DistortionVolume:
 		return kickDistortionVolumeEnvelope;
-	else
+	default:
 		return {};
+	}
 }
 
-std::shared_ptr<PercussionState::OscillatorInfo>
-PercussionState::getOscillator(int index) const
+PercussionState::OscillatorInfo*
+PercussionState::getOscillator(int index)
 {
         index += GKICK_OSC_GROUP_SIZE * static_cast<int>(currentLayer);
         auto it = oscillators.find(index);
         if (it != oscillators.end())
-                return it->second;
+                return &it->second;
+
+        return nullptr;
+}
+
+const PercussionState::OscillatorInfo*
+PercussionState::getConstOscillator(int index) const
+{
+        index += GKICK_OSC_GROUP_SIZE * static_cast<int>(currentLayer);
+        auto it = oscillators.find(index);
+        if (it != oscillators.end())
+                return &it->second;
 
         return nullptr;
 }
@@ -613,15 +694,56 @@ void PercussionState::setOscillatorEnvelopePoints(int index,
                 case GeonkickApi::EnvelopeType::FilterCutOff:
                         oscillator->filterCutOffEnvelope = points;
                         break;
+		case GeonkickApi::EnvelopeType::FilterQFactor:
+                        oscillator->filterQFactorEnvelope = points;
+                        break;
                 default:
                         return;
                 }
         }
 }
 
-bool PercussionState::isOscillatorAsFm(int index) const
+void PercussionState::setOscillatorEnvelopeApplyType(int index,
+						     GeonkickApi::EnvelopeType envelopeType,
+						     GeonkickApi::EnvelopeApplyType applyType)
 {
         auto oscillator = getOscillator(index);
+        if (!oscillator)
+                return;
+
+	switch (envelopeType) {
+	case GeonkickApi::EnvelopeType::Frequency:
+		oscillator->frequencyEnvelopeApplyType = applyType;
+		break;
+	case GeonkickApi::EnvelopeType::FilterCutOff:
+		oscillator->cutOffEnvelopeApplyType = applyType;
+		break;
+	default:
+		return;
+	}
+}
+
+GeonkickApi::EnvelopeApplyType PercussionState::getOscillatorEnvelopeApplyType(int index,
+									       GeonkickApi::EnvelopeType envelope) const
+{
+        auto oscillator = getConstOscillator(index);
+        if (!oscillator)
+                return GeonkickApi::EnvelopeApplyType::Linear;
+
+	switch (envelope) {
+	case GeonkickApi::EnvelopeType::Frequency:
+		return oscillator->frequencyEnvelopeApplyType;
+		break;
+	case GeonkickApi::EnvelopeType::FilterCutOff:
+		return oscillator->cutOffEnvelopeApplyType;
+	default:
+		return GeonkickApi::EnvelopeApplyType::Linear;
+	}
+}
+
+bool PercussionState::isOscillatorAsFm(int index) const
+{
+        auto oscillator = getConstOscillator(index);
         if (oscillator)
                 return oscillator->isFm;
         return false;
@@ -636,7 +758,7 @@ void PercussionState::setOscillatorAsFm(int index, bool b)
 
 bool PercussionState::isOscillatorEnabled(int index) const
 {
-        auto oscillator = getOscillator(index);
+        auto oscillator = getConstOscillator(index);
         if (oscillator)
                 return oscillator->isEnabled;
 
@@ -645,7 +767,7 @@ bool PercussionState::isOscillatorEnabled(int index) const
 
 GeonkickApi::FunctionType PercussionState::oscillatorFunction(int index) const
 {
-        auto oscillator = getOscillator(index);
+        auto oscillator = getConstOscillator(index);
         if (oscillator)
                 return oscillator->function;
 
@@ -654,7 +776,7 @@ GeonkickApi::FunctionType PercussionState::oscillatorFunction(int index) const
 
 double PercussionState::oscillatorPhase(int index) const
 {
-        auto oscillator = getOscillator(index);
+        auto oscillator = getConstOscillator(index);
         if (oscillator)
                 return oscillator->phase;
 
@@ -663,7 +785,7 @@ double PercussionState::oscillatorPhase(int index) const
 
 int PercussionState::oscillatorSeed(int index) const
 {
-        auto oscillator = getOscillator(index);
+        auto oscillator = getConstOscillator(index);
         if (oscillator)
                 return oscillator->seed;
 
@@ -672,35 +794,35 @@ int PercussionState::oscillatorSeed(int index) const
 
 double PercussionState::oscillatorAmplitue(int index) const
 {
-        if (auto oscillator = getOscillator(index); oscillator)
+        if (auto oscillator = getConstOscillator(index); oscillator)
                 return oscillator->amplitude;
         return 0;
 }
 
 double PercussionState::oscillatorFrequency(int index) const
 {
-        if (auto oscillator = getOscillator(index); oscillator)
+        if (auto oscillator = getConstOscillator(index); oscillator)
                 return oscillator->frequency;
         return 0;
 }
 
 double PercussionState::oscillatorPitchShift(int index) const
 {
-        if (auto oscillator = getOscillator(index); oscillator)
+        if (auto oscillator = getConstOscillator(index); oscillator)
                 return oscillator->pitchShift;
         return 0;
 }
 
 bool PercussionState::isOscillatorFilterEnabled(int index) const
 {
-        if (auto oscillator = getOscillator(index); oscillator)
+        if (auto oscillator = getConstOscillator(index); oscillator)
                 return oscillator->isFilterEnabled;
         return false;
 }
 
 GeonkickApi::FilterType PercussionState::oscillatorFilterType(int index) const
 {
-        auto oscillator = getOscillator(index);
+        auto oscillator = getConstOscillator(index);
         if (oscillator)
                 return oscillator->filterType;
         return GeonkickApi::FilterType::LowPass;
@@ -708,7 +830,7 @@ GeonkickApi::FilterType PercussionState::oscillatorFilterType(int index) const
 
 double PercussionState::oscillatorFilterCutOffFreq(int index) const
 {
-        auto oscillator = getOscillator(index);
+        auto oscillator = getConstOscillator(index);
         if (oscillator)
                 return oscillator->filterFrequency;
         return 0;
@@ -716,7 +838,7 @@ double PercussionState::oscillatorFilterCutOffFreq(int index) const
 
 double PercussionState::oscillatorFilterFactor(int index) const
 {
-        auto oscillator = getOscillator(index);
+        auto oscillator = getConstOscillator(index);
         if (oscillator)
                 return oscillator->filterFactor;
         return 0;
@@ -725,7 +847,7 @@ double PercussionState::oscillatorFilterFactor(int index) const
 std::vector<RkRealPoint>
 PercussionState::oscillatorEnvelopePoints(int index, GeonkickApi::EnvelopeType type) const
 {
-        if (auto oscillator = getOscillator(index); oscillator) {
+        if (auto oscillator = getConstOscillator(index); oscillator) {
                 switch (type) {
                 case GeonkickApi::EnvelopeType::Amplitude:
                         return oscillator->amplitudeEnvelope;
@@ -735,6 +857,8 @@ PercussionState::oscillatorEnvelopePoints(int index, GeonkickApi::EnvelopeType t
                         return oscillator->pitchShiftEnvelope;
                 case GeonkickApi::EnvelopeType::FilterCutOff:
                         return oscillator->filterCutOffEnvelope;
+		case GeonkickApi::EnvelopeType::FilterQFactor:
+                        return oscillator->filterQFactorEnvelope;
                 default:
                         return {};
                 }
@@ -866,10 +990,12 @@ std::string PercussionState::toJson() const
 void PercussionState::envelopeToJson(std::ostringstream &jsonStream,
                                      const std::string &envName,
                                      double amplitude,
-                                     const std::vector<RkRealPoint> &envelope)
+                                     const std::vector<RkRealPoint> &envelope,
+ 				     GeonkickApi::EnvelopeApplyType applyType)
 {
         jsonStream << "\"" << envName << "\": {" << std::endl;
         jsonStream << "\"amplitude\": " << amplitude << ", " << std::endl;
+	jsonStream << "\"apply_type\": " << static_cast<int>(applyType) << ", " << std::endl;
         jsonStream << "\"points\": [" << std::endl;
         bool first = true;
         for (const auto &point: envelope) {
@@ -887,38 +1013,41 @@ void PercussionState::oscJson(std::ostringstream &jsonStream) const
 {
         for (const auto& val: oscillators) {
                 jsonStream << "\"osc" << val.first << "\": {" << std::endl;
-                jsonStream << "\"enabled\": " << (val.second->isEnabled ? "true" : "false") << ", " << std::endl;
-                jsonStream << "\"is_fm\": " << (val.second->isFm ? "true" : "false") << ", " << std::endl;
-                if (val.second->function == GeonkickApi::FunctionType::Sample && !val.second->sample.empty())
-                        jsonStream <<  "\"sample\": \"" << toBase64F(val.second->sample) << "\"," << std::endl;
-                jsonStream <<  "\"function\": " << static_cast<int>(val.second->function) << "," << std::endl;
+                jsonStream << "\"enabled\": " << (val.second.isEnabled ? "true" : "false") << ", " << std::endl;
+                jsonStream << "\"is_fm\": " << (val.second.isFm ? "true" : "false") << ", " << std::endl;
+                if (val.second.function == GeonkickApi::FunctionType::Sample && !val.second.sample.empty())
+                        jsonStream <<  "\"sample\": \"" << toBase64F(val.second.sample) << "\"," << std::endl;
+                jsonStream <<  "\"function\": " << static_cast<int>(val.second.function) << "," << std::endl;
                 jsonStream <<  "\"phase\": " 
-                           << val.second->phase << ", " << std::endl;
-                jsonStream <<  "\"seed\": " << val.second->seed << ", " << std::endl;
+                           << val.second.phase << ", " << std::endl;
+                jsonStream <<  "\"seed\": " << val.second.seed << ", " << std::endl;
                 envelopeToJson(jsonStream,
                                "ampl_env",
-                               val.second->amplitude,
-                               val.second->amplitudeEnvelope);
+                               val.second.amplitude,
+                               val.second.amplitudeEnvelope);
                 jsonStream << "," << std::endl;
                 envelopeToJson(jsonStream,
                                "freq_env",
-                               val.second->frequency,
-                               val.second->frequencyEnvelope);
+                               val.second.frequency,
+                               val.second.frequencyEnvelope,
+			       val.second.frequencyEnvelopeApplyType);
                 jsonStream << "," << std::endl;
                 envelopeToJson(jsonStream,
                                "pitchshift_env",
-                               val.second->pitchShift,
-                               val.second->pitchShiftEnvelope);
+                               val.second.pitchShift,
+                               val.second.pitchShiftEnvelope);
                 jsonStream << "," << std::endl;
                 jsonStream << "\"filter\": {" << std::endl;
-                jsonStream << "\"enabled\": " << (val.second->isFilterEnabled ? "true" : "false");
+                jsonStream << "\"enabled\": " << (val.second.isFilterEnabled ? "true" : "false");
                 jsonStream << ", " << std::endl;
-                jsonStream << "\"type\": " << static_cast<int>(val.second->filterType) << ", " << std::endl;
+                jsonStream << "\"type\": " << static_cast<int>(val.second.filterType) << ", " << std::endl;
                 jsonStream << "\"cutoff\": " 
-                           << val.second->filterFrequency << ", " << std::endl;
+                           << val.second.filterFrequency << ", " << std::endl;
+		jsonStream << "\"apply_type\": " 
+                           << static_cast<int>(val.second.cutOffEnvelopeApplyType) << ", " << std::endl;
                 jsonStream << "\"cutoff_env\": [";
                 bool first = true;
-                for (const auto &point: val.second->filterCutOffEnvelope) {
+                for (const auto &point: val.second.filterCutOffEnvelope) {
                         if (first)
                                 first = false;
                         else
@@ -928,7 +1057,18 @@ void PercussionState::oscJson(std::ostringstream &jsonStream) const
                 }
                 jsonStream << "], " << std::endl;
                 jsonStream << "\"factor\": " 
-                           << val.second->filterFactor << std::endl;
+                           << val.second.filterFactor << "," << std::endl;
+		jsonStream << "\"qfactor_env\": [";
+                first = true;
+                for (const auto &point: val.second.filterQFactorEnvelope) {
+                        if (first)
+                                first = false;
+                        else
+                                jsonStream << ", ";
+                        jsonStream << "[ "  << point.x()
+                                   << " , "  << point.y() << "]";
+                }
+		jsonStream << "] " << std::endl;
                 jsonStream << "}" << std::endl;  // filter;
                 jsonStream << "}" << std::endl;  // osc;
                 jsonStream << "," << std::endl;
@@ -941,6 +1081,7 @@ void PercussionState::kickJson(std::ostringstream &jsonStream) const
         jsonStream << "\"PercussionAppVersion\": " << GEONKICK_VERSION << "," << std::endl;
 	jsonStream << "\"id\": " << getId() << "," << std::endl;
         jsonStream << "\"channel\": " << getChannel() << "," << std::endl;
+        jsonStream << "\"midiChannel\": " << static_cast<int>(getMidiChannel()) << "," << std::endl;
         jsonStream << "\"mute\": " << (isMuted() ? "true" : "false") << "," << std::endl;
         jsonStream << "\"solo\": " << (isSolo() ? "true" : "false") << "," << std::endl;
         jsonStream << "\"name\": \"" << getName() << "\"," << std::endl;
@@ -994,6 +1135,8 @@ void PercussionState::kickJson(std::ostringstream &jsonStream) const
         jsonStream << "\"type\": " << static_cast<int>(getKickFilterType()) << ", " << std::endl;
         jsonStream << "\"cutoff\": "
                    << getKickFilterFrequency() << ", " << std::endl;
+	jsonStream << "\"apply_type\": "
+                   << static_cast<int>(getKickEnvelopeApplyType(GeonkickApi::EnvelopeType::FilterCutOff)) << ", " << std::endl;
         jsonStream << "\"factor\": "
                    << getKickFilterQFactor() << ", " << std::endl;
         points = getKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterCutOff);
@@ -1007,6 +1150,18 @@ void PercussionState::kickJson(std::ostringstream &jsonStream) const
                 jsonStream << "[ "  << point.x()
                            << " , "  << point.y() << "]";
         }
+        jsonStream << "]," << std::endl; // points
+	points = getKickEnvelopePoints(GeonkickApi::EnvelopeType::FilterQFactor);
+        jsonStream << "\"qfactor_env\": [";
+        first = true;
+        for (const auto &point: points) {
+                if (first)
+                        first = false;
+                else
+                        jsonStream << ", ";
+                jsonStream << "[ "  << point.x()
+                           << " , "  << point.y() << "]";
+        }
         jsonStream << "]" << std::endl; // points
         jsonStream << "}, " << std::endl;  // filter;
         jsonStream << "\"compressor\": {" << std::endl;
@@ -1150,6 +1305,13 @@ void PercussionState::setOscillatorSample(int oscillatorIndex, const std::vector
 
 std::vector<float> PercussionState::getOscillatorSample(int oscillatorIndex) const
 {
-        auto oscillator = getOscillator(oscillatorIndex);
+        auto oscillator = getConstOscillator(oscillatorIndex);
         return oscillator->sample;
 }
+
+GeonkickApi::EnvelopeApplyType PercussionState::getApplyTypeFromObj(const rapidjson::Value &value) const
+{
+	if (value.IsInt())
+		return static_cast<GeonkickApi::EnvelopeApplyType>(value.GetInt());
+	return GeonkickApi::EnvelopeApplyType::Linear;
+}
