@@ -38,11 +38,13 @@ class PercussionState
         PercussionState();
         bool loadFile(const std::string &file);
         bool loadData(const std::string &data);
-        void loadObject(const rapidjson::Value &obj);
+        bool loadObject(const rapidjson::Value &obj);
         size_t getId() const;
         void setId(size_t id);
         void setChannel(size_t channel);
         size_t getChannel() const;
+        void setMidiChannel(signed char channel);
+        signed char getMidiChannel() const;
         void setMute(bool b);
         bool isMuted() const;
         void setSolo(bool b);
@@ -62,6 +64,9 @@ class PercussionState
         void setKickFilterType(GeonkickApi::FilterType type);
         void setKickEnvelopePoints(GeonkickApi::EnvelopeType envelope,
                                    const std::vector<RkRealPoint> &points);
+	void setKickEnvelopeApplyType(GeonkickApi::EnvelopeType envelope,
+				      GeonkickApi::EnvelopeApplyType applyType);
+
         double getLimiterValue() const;
         double getKickLength() const;
         double getKickAmplitude() const;
@@ -70,6 +75,7 @@ class PercussionState
         double getKickFilterQFactor() const;
         GeonkickApi::FilterType getKickFilterType() const;
         std::vector<RkRealPoint> getKickEnvelopePoints(GeonkickApi::EnvelopeType envelope) const;
+	GeonkickApi::EnvelopeApplyType getKickEnvelopeApplyType(GeonkickApi::EnvelopeType envelope) const;
         void setOscillatorEnabled(int index, bool b);
         void setOscillatorFunction(int index, GeonkickApi::FunctionType type);
         void setOscillatorPhase(int index, double phase);
@@ -81,6 +87,11 @@ class PercussionState
         void setOscillatorFilterType(int index, GeonkickApi::FilterType type);
         void setOscillatorFilterCutOffFreq(int index, double val);
         void setOscillatorFilterFactor(int index, double val);
+	void setOscillatorEnvelopeApplyType(int index,
+					    GeonkickApi::EnvelopeType envelopeType,
+					    GeonkickApi::EnvelopeApplyType applyType);
+	GeonkickApi::EnvelopeApplyType getOscillatorEnvelopeApplyType(int index,
+								      GeonkickApi::EnvelopeType envelope) const;
         void setOscillatorEnvelopePoints(int index,
                                          const std::vector<RkRealPoint> &points,
                                          GeonkickApi::EnvelopeType envelope);
@@ -142,9 +153,11 @@ class PercussionState
         static void envelopeToJson(std::ostringstream &jsonStream,
                                    const std::string &envName,
                                    double amplitude,
-                                   const std::vector<RkRealPoint> &envelope);
+                                   const std::vector<RkRealPoint> &envelope,
+				   GeonkickApi::EnvelopeApplyType applyType = GeonkickApi::EnvelopeApplyType::Linear);
         void oscJson(std::ostringstream &jsonStream) const;
         void kickJson(std::ostringstream &jsonStream) const;
+	GeonkickApi::EnvelopeApplyType getApplyTypeFromObj(const rapidjson::Value &obj) const;
 
 private:
         void initOscillators();
@@ -162,7 +175,9 @@ private:
                 , isFilterEnabled{false}
                 , filterType{GeonkickApi::FilterType::LowPass}
                 , filterFrequency{200}
-                , filterFactor{1.0} {}
+                , filterFactor{1.0}
+	        , frequencyEnvelopeApplyType{GeonkickApi::EnvelopeApplyType::Logarithmic}
+	        , cutOffEnvelopeApplyType{GeonkickApi::EnvelopeApplyType::Logarithmic} {}
                 GeonkickApi::OscillatorType type;
                 std::vector<float> sample;
                 bool isEnabled;
@@ -178,12 +193,16 @@ private:
                 double filterFrequency;
                 double filterFactor;
                 std::vector<RkRealPoint> amplitudeEnvelope;
+		GeonkickApi::EnvelopeApplyType frequencyEnvelopeApplyType;
                 std::vector<RkRealPoint> frequencyEnvelope;
+		GeonkickApi::EnvelopeApplyType cutOffEnvelopeApplyType;
                 std::vector<RkRealPoint> filterCutOffEnvelope;
+		std::vector<RkRealPoint> filterQFactorEnvelope;
                 std::vector<RkRealPoint> pitchShiftEnvelope;
         };
 
-        std::shared_ptr<OscillatorInfo> getOscillator(int index) const;
+        OscillatorInfo* getOscillator(int index);
+        const OscillatorInfo* getConstOscillator(int index) const;
 
         struct Compressor {
                 bool enabled;
@@ -207,6 +226,7 @@ private:
         std::string kickName;
         signed char playingKey;
         size_t outputChannel;
+        signed char outputMidiChannel;
         bool kickEnabled;
         bool percussionMuted;
         bool percussionSolo;
@@ -218,11 +238,13 @@ private:
         double kickFilterFrequency;
         double kickFilterQFactor;
         GeonkickApi::FilterType kickFilterType;
+	GeonkickApi::EnvelopeApplyType kickCutOffEnvelopeApplyType;
         std::vector<RkRealPoint> kickFilterCutOffEnvelope;
+	std::vector<RkRealPoint> kickFilterQFactorEnvelope;
         std::vector<RkRealPoint> kickEnvelopePoints;
 	std::vector<RkRealPoint> kickDistortionDriveEnvelope;
         std::vector<RkRealPoint> kickDistortionVolumeEnvelope;
-        std::unordered_map<int, std::shared_ptr<OscillatorInfo>> oscillators;
+        std::unordered_map<int, OscillatorInfo> oscillators;
         Compressor compressor;
         Distortion distortion;
         std::vector<bool> layers;
