@@ -34,6 +34,7 @@
 #include <RkButton.h>
 #include <RkContainer.h>
 #include <RkProgressBar.h>
+#include <RkSpinBox.h>
 
 RK_DECLARE_IMAGE_RC(mute);
 RK_DECLARE_IMAGE_RC(mute_hover);
@@ -101,6 +102,7 @@ KitPercussionView::KitPercussionView(KitWidget *parent,
         , nameWidth{100}
         , channelWidth{30}
         , editPercussion{nullptr}
+        , midiChannelSpinBox{nullptr}
         , keyButton{nullptr}
         , copyButton{nullptr}
         , removeButton{nullptr}
@@ -109,7 +111,7 @@ KitPercussionView::KitPercussionView(KitWidget *parent,
         , soloButton{nullptr}
         , percussionLimiter{nullptr}
 {
-        setSize(parent->width(), 20);
+        setSize(parent->width(), 21);
         createView();
         setModel(model);
 }
@@ -126,9 +128,32 @@ void KitPercussionView::createView()
         auto percussionContainer = new RkContainer(this);
         percussionContainer->setSize(size());
         percussionContainer->setHiddenTakesPlace();
-        percussionContainer->addSpace(nameWidth + percussionModel->numberOfChannels() * channelWidth + 10);
+        percussionContainer->addSpace(nameWidth + percussionModel->numberOfChannels() * channelWidth + 5);
 
-        // Midi key button
+        // Midi channel spinbox.
+        midiChannelSpinBox = new RkSpinBox(this);
+                midiChannelSpinBox->setTextColor({250, 250, 250});
+        midiChannelSpinBox->setBackgroundColor({60, 57, 57});
+        midiChannelSpinBox->upControl()->setBackgroundColor({50, 47, 47});
+        midiChannelSpinBox->upControl()->setTextColor({100, 100, 100});
+        midiChannelSpinBox->downControl()->setBackgroundColor({50, 47, 47});
+        midiChannelSpinBox->downControl()->setTextColor({100, 100, 100});
+        midiChannelSpinBox->setSize(50, 20);
+        midiChannelSpinBox->show();
+        RK_ACT_BIND(midiChannelSpinBox,
+                    currentIndexChanged,
+                    RK_ACT_ARGS(int index),
+                    percussionModel,
+                    setMidiChannel(index - 1));
+        RK_ACT_BIND(percussionModel,
+                    midiChannelUpdated,
+                    RK_ACT_ARGS(int index),
+                    midiChannelSpinBox,
+                    setCurrentIndex(index + 1));
+        percussionContainer->addWidget(midiChannelSpinBox);
+        percussionContainer->addSpace(5);
+
+        // Midi key button.
         keyButton = new GeonkickButton(this);
         keyButton->setTextColor({250, 250, 250});
         keyButton->setType(RkButton::ButtonType::ButtonUncheckable);
@@ -238,6 +263,11 @@ void KitPercussionView::updateView()
         percussionLimiter->onSetValue(percussionModel->limiter());
         muteButton->setPressed(percussionModel->isMuted());
         soloButton->setPressed(percussionModel->isSolo());
+        size_t nMidiChannels = percussionModel->numberOfMidiChannels();
+        midiChannelSpinBox->addItem("Any");
+        for (size_t i = 0; i < nMidiChannels; i++)
+                midiChannelSpinBox->addItem(std::to_string(i + 1));
+        midiChannelSpinBox->setCurrentIndex(percussionModel->midiChannel() + 1);
         keyButton->setText(MidiKeyWidget::midiKeyToNote(percussionModel->key()));
         keyButton->setBackgroundColor((index() % 2) ? RkColor(100, 100, 100) : RkColor(50, 50, 50));
         update();
@@ -263,6 +293,7 @@ void KitPercussionView::setModel(PercussionModel *model)
         RK_ACT_BIND(percussionModel, soloUpdated, RK_ACT_ARGS(bool b), soloButton, setPressed(b));
         RK_ACT_BIND(percussionModel, selected, RK_ACT_ARGS(), this, update());
         RK_ACT_BIND(percussionModel, modelUpdated, RK_ACT_ARGS(), this, updateView());
+        RK_ACT_BIND(percussionModel, midiChannelUpdated, RK_ACT_ARGS(int val), this, update());
         updateView();
 }
 
@@ -337,10 +368,8 @@ void KitPercussionView::mouseButtonPressEvent(RkMouseEvent *event)
                 int rightLimit = nameWidth + channelWidth * percussionModel->numberOfChannels();
                 if (event->x() <= leftLimit)
                         percussionModel->select();
-                else if (event->x() > leftLimit && event->x() < rightLimit) {
-                        GEONKICK_LOG_INFO("set channel: " << (event->x() - nameWidth) / channelWidth);
+                else if (event->x() > leftLimit && event->x() < rightLimit)
                         percussionModel->setChannel((event->x() - nameWidth) / channelWidth);
-                }
         }
 }
 
