@@ -29,13 +29,6 @@
 
 #include <RkLabel.h>
 
-class RkLineEdit;
-class RkButton;
-class PercussionModel;
-class RkProgressBar;
-class KitWidget;
-class GeonkickButton;
-
 class PercussionLimiter : public GeonkickSlider {
   public:
         PercussionLimiter(GeonkickWidget *parent);
@@ -49,6 +42,14 @@ class PercussionLimiter : public GeonkickSlider {
         int levelerValue;
 };
 
+class RkLineEdit;
+class RkSpinBox;
+class RkButton;
+class PercussionModel;
+class RkProgressBar;
+class KitWidget;
+class GeonkickButton;
+
 class KitPercussionView: public GeonkickWidget
 {
  public:
@@ -79,6 +80,7 @@ class KitPercussionView: public GeonkickWidget
         int nameWidth;
         int channelWidth;
         RkLineEdit *editPercussion;
+        RkSpinBox *midiChannelSpinBox;
         GeonkickButton *keyButton;
         RkButton *copyButton;
         RkButton *removeButton;
