@@ -35,7 +35,7 @@ class RK_EXPORT RkObserver {
                 : observerObject{obj} { }
         virtual ~RkObserver() = default;
         RkObject *object() { return observerObject; }
-        void setObject(RkObject *obj) { obj = observerObject; }
+        void setObject(RkObject *obj) { observerObject = obj; }
  private:
         RkObject *observerObject;
 };
