@@ -78,13 +78,18 @@ RK_DECLARE_IMAGE_RC(topmenu_presets_off);
 RK_DECLARE_IMAGE_RC(topmenu_samples_active);
 RK_DECLARE_IMAGE_RC(topmenu_samples_hover);
 RK_DECLARE_IMAGE_RC(topmenu_samples_off);
+RK_DECLARE_IMAGE_RC(topmenu_settings_active);
+RK_DECLARE_IMAGE_RC(topmenu_settings_hover);
+RK_DECLARE_IMAGE_RC(topmenu_settings_off);
 
 TopBar::TopBar(GeonkickWidget *parent, KitModel *model)
         : GeonkickWidget(parent)
         , kitModel{model}
         , openFileButton{nullptr}
         , saveFileButton{nullptr}
+#ifndef GEONKICK_OS_WINDOWS
         , exportFileButton{nullptr}
+#endif // GEONKICK_OS_WINDOWS
         , presetNameLabel{nullptr}
         , layer1Button{nullptr}
         , layer2Button{nullptr}
@@ -97,14 +102,15 @@ TopBar::TopBar(GeonkickWidget *parent, KitModel *model)
         , presetsButton{nullptr}
         , samplesButton{nullptr}
 {
+        setName("TopBar");
         setFixedSize({parent->width(), 30});
         auto mainLayout = new RkContainer(this);
         mainLayout->setSize(size());
 
         auto logo = new RkLabel(this);
-        RkImage image(62, 10, RK_IMAGE_RC(logo));
+        logo->setBackgroundColor(background());
+        RkImage image(22, 22, RK_IMAGE_RC(logo));
         logo->setSize(image.width(), image.height());
-        logo->setBackgroundColor(68, 68, 70);
         logo->setImage(image);
         logo->show();
         mainLayout->addWidget(logo);
@@ -120,7 +126,7 @@ TopBar::TopBar(GeonkickWidget *parent, KitModel *model)
                                  RkButton::State::UnpressedHover);
         openFileButton->setImage(RkImage(openFileButton->size(), RK_IMAGE_RC(open_hover)),
                                  RkButton::State::Pressed);
-        RK_ACT_BINDL(openFileButton, pressed, RK_ACT_ARGS(), [=](){openFileButton->setPressed(false);
+        RK_ACT_BINDL(openFileButton, pressed, RK_ACT_ARGS(), [=,this](){openFileButton->setPressed(false);
                                                               action openFile();});
         mainLayout->addWidget(openFileButton);
         addSeparator(mainLayout, 10);
@@ -134,11 +140,11 @@ TopBar::TopBar(GeonkickWidget *parent, KitModel *model)
                                  RkButton::State::UnpressedHover);
         saveFileButton->setImage(RkImage(saveFileButton->size(), RK_IMAGE_RC(save_hover)),
                                  RkButton::State::Pressed);
-        RK_ACT_BINDL(saveFileButton, pressed, RK_ACT_ARGS(), [=](){saveFileButton->setPressed(false);
+        RK_ACT_BINDL(saveFileButton, pressed, RK_ACT_ARGS(), [=,this](){saveFileButton->setPressed(false);
                                                              action saveFile();});
         mainLayout->addWidget(saveFileButton);
         addSeparator(mainLayout, 10);
-
+#ifndef GEONKICK_OS_WINDOWS
         exportFileButton = new GeonkickButton(this);
         exportFileButton->setSize(29, 10);
         saveFileButton->setType(RkButton::ButtonType::ButtonCheckable);
@@ -148,10 +154,11 @@ TopBar::TopBar(GeonkickWidget *parent, KitModel *model)
                                  RkButton::State::UnpressedHover);
         exportFileButton->setImage(RkImage(exportFileButton->size(), RK_IMAGE_RC(export_hover)),
                                 RkButton::State::Pressed);
-        RK_ACT_BINDL(exportFileButton, pressed, RK_ACT_ARGS(), [=](){exportFileButton->setPressed(false);
+        RK_ACT_BINDL(exportFileButton, pressed, RK_ACT_ARGS(), [=,this](){exportFileButton->setPressed(false);
                                                                action openExport();});
         mainLayout->addWidget(exportFileButton);
         addSeparator(mainLayout, 10);
+#endif // GEONKICK_OS_WINDOWS
 
 	auto playButton = new RkButton(this);
         playButton->setType(RkButton::ButtonType::ButtonPush);
@@ -291,11 +298,11 @@ TopBar::TopBar(GeonkickWidget *parent, KitModel *model)
         samplesButton->setPressed(viewState()->getMainView() == ViewState::View::Samples);
         samplesButton->setFixedSize(54, 20);
         samplesButton->setImage(RkImage(samplesButton->size(), RK_IMAGE_RC(topmenu_samples_off)),
-                               RkButton::State::Unpressed);
+                                RkButton::State::Unpressed);
         samplesButton->setImage(RkImage(samplesButton->size(), RK_IMAGE_RC(topmenu_samples_active)),
-                               RkButton::State::Pressed);
+                                RkButton::State::Pressed);
         samplesButton->setImage(RkImage(samplesButton->size(), RK_IMAGE_RC(topmenu_samples_hover)),
-                               RkButton::State::UnpressedHover);
+                                RkButton::State::UnpressedHover);
         samplesButton->show();
         RK_ACT_BIND(samplesButton, pressed, RK_ACT_ARGS(),
                     viewState(), setMainView(ViewState::View::Samples));
@@ -303,9 +310,30 @@ TopBar::TopBar(GeonkickWidget *parent, KitModel *model)
                     samplesButton, setPressed(view == ViewState::View::Samples));
         mainLayout->addWidget(samplesButton);
 
+        // Setting button
+        addSeparator(mainLayout);
+        auto settingsButton = new GeonkickButton(this);
+        settingsButton->setPressed(viewState()->getMainView() == ViewState::View::Samples);
+        settingsButton->setFixedSize(54, 20);
+        settingsButton->setImage(RkImage(settingsButton->size(),
+                                         RK_IMAGE_RC(topmenu_settings_off)),
+                                 RkButton::State::Unpressed);
+        settingsButton->setImage(RkImage(settingsButton->size(),
+                                         RK_IMAGE_RC(topmenu_settings_active)),
+                                 RkButton::State::Pressed);
+        settingsButton->setImage(RkImage(settingsButton->size(),
+                                         RK_IMAGE_RC(topmenu_settings_hover)),
+                                 RkButton::State::UnpressedHover);
+        settingsButton->show();
+        RK_ACT_BIND(settingsButton, pressed, RK_ACT_ARGS(),
+                    viewState(), setMainView(ViewState::View::Settings));
+        RK_ACT_BIND(viewState(), mainViewChanged, RK_ACT_ARGS(ViewState::View view),
+                    settingsButton, setPressed(view == ViewState::View::Settings));
+        mainLayout->addWidget(settingsButton);
+
         RK_ACT_BIND(kitModel, modelUpdated, RK_ACT_ARGS(), this, updateGui());
         RK_ACT_BINDL(kitModel, percussionUpdated, RK_ACT_ARGS(PercussionModel* model),
-                     [=](PercussionModel* model) {
+                     [=, this](PercussionModel* model) {
                              if (model->isSelected())
                                      updateGui();
                      } );
@@ -378,7 +406,7 @@ void TopBar::createLyersButtons(RkContainer *mainLayout)
         RK_ACT_BIND(layer3Button, toggled, RK_ACT_ARGS(bool b),
                     kitModel->api(), enbaleLayer(GeonkickApi::Layer::Layer3, b));
         RK_ACT_BIND(layer2Button, toggled, RK_ACT_ARGS(bool b),
-                    kitModel->api(), enbaleLayer(GeonkickApi::Layer::Layer2, b));
+        kitModel->api(), enbaleLayer(GeonkickApi::Layer::Layer2, b));
 }
 
 void TopBar::setPresetName(const std::string &name)
