@@ -42,7 +42,9 @@ class TopBar : public GeonkickWidget
         RK_DECL_ACT(openFile, openFile(), RK_ARG_TYPE(), RK_ARG_VAL());
         RK_DECL_ACT(openAbout, openAbout(), RK_ARG_TYPE(), RK_ARG_VAL());
         RK_DECL_ACT(resetToDefault, resetToDefault(), RK_ARG_TYPE(), RK_ARG_VAL());
+#ifndef GEONKICK_OS_WINDOWS
         RK_DECL_ACT(openExport, openExport(), RK_ARG_TYPE(), RK_ARG_VAL());
+#endif // GEONKICK_OS_WINDOWS
         RK_DECL_ACT(layerSelected,
                     layerSelected(GeonkickApi::Layer layer, bool b),
                     RK_ARG_TYPE(GeonkickApi::Layer, bool),
@@ -58,7 +60,9 @@ class TopBar : public GeonkickWidget
         KitModel *kitModel;
         GeonkickButton *openFileButton;
         GeonkickButton *saveFileButton;
+#ifndef GEONKICK_OS_WINDOWS
         GeonkickButton *exportFileButton;
+#endif // GEONKICK_OS_WINDOWS
         RkLabel* presetNameLabel;
         GeonkickButton *layer1Button;
         GeonkickButton *layer2Button;
